import 'cypress-real-events';
import { exist } from '../../support/cypress.util';

export class BenutzerE2EComponent {
  private readonly benutzerHinzufuegenButton: string = 'Add-user-button';
  private readonly userEntry: string = 'User-entry-';
  private readonly userVorname: string = 'Vorname-text-input';
  private readonly userNachname: string = 'Nachname-text-input';
  private readonly userBenutzername: string = 'Benutzername-text-input';
  private readonly userMail: string = 'E-Mail-text-input';
  private readonly addOEButton: string = 'Add-organisationseinheit-button';
  private readonly adminCheckbox: string = 'Admin-checkbox-editor';
  private readonly loeschenCheckbox: string = 'Loschen-checkbox-editor';
  private readonly userCheckbox: string = 'User-checkbox-editor';
  private readonly postCheckbox: string = 'Poststelle-checkbox-editor';
  private readonly userRolesTable: string = 'User-roles-table';

  public getHinzufuegenButton(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.benutzerHinzufuegenButton);
  }

  public getUserRolesTable(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.userRolesTable);
  }

  public addUser(): void {
    this.getHinzufuegenButton().click();
  }

  public getUserEntry(user: string): Cypress.Chainable<Element> {
    user = this.userEntry + user;
    return cy.getTestElement(user);
  }

  public stringExistsInUserEntry(phrase: string, user: string): void {
    this.getUserEntry(user).within(() => {
      exist(cy.contains(phrase));
    });
  }

  public getVornameInput(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.userVorname);
  }

  public getNachnameInput(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.userNachname);
  }

  public getBenutzernameInput(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.userBenutzername);
  }

  public getMailInput(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.userMail);
  }

  public getOEButton(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.addOEButton);
  }

  public addOrganisationseinheit(): void {
    this.getOEButton().click();
  }

  public getAdminCheckbox(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.adminCheckbox);
  }

  public clickAdminCheckbox(): void {
    this.getAdminCheckbox().click();
  }

  public getLoeschenCheckbox() {
    return cy.getTestElement(this.loeschenCheckbox);
  }

  public clickLoeschenCheckbox(): void {
    this.getLoeschenCheckbox().click();
  }

  public getUserCheckbox(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.userCheckbox);
  }

  public clickUserCheckbox(): void {
    this.getUserCheckbox().click();
  }

  public getPostCheckbox(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.postCheckbox);
  }

  public clickPostCheckbox(): void {
    this.getPostCheckbox().click();
  }
}
