import { MainPage } from 'apps/admin-e2e/src/page-objects/main.po';
import { BenutzerE2EComponent } from '../../../components/benutzer/benutzer.e2e.component';
import { beChecked, beEnabled, exist, notBeChecked, notBeEnabled } from '../../../support/cypress.util';
import { loginAsAriane } from '../../../support/user-util';

const mainPage: MainPage = new MainPage();
const benutzerPage: BenutzerE2EComponent = new BenutzerE2EComponent();
const role1: string = 'VERWALTUNG_USER';
const role2: string = 'VERWALTUNG_LOESCHEN';
const role3: string = 'VERWALTUNG_POSTSTELLE';
const orga1: string = 'Ordnungsamt';
const orga2: string = 'Denkmalpflege';
const orga3: string = 'Wirtschaftsförderung';
const orga_none: string = 'keine zuständige Stelle zugewiesen';
const mail1: string = 'peter.von.der.post@ozg-sh.de';

describe('Benutzer und Rollen', () => {
  before(() => {
    loginAsAriane();
  });

  it('should open Benutzer tab and show Hinzufuegen button', () => {
    mainPage.clickBenutzerTab();

    exist(benutzerPage.getHinzufuegenButton());
  });

  it('should show table with users and roles', () => {
    exist(benutzerPage.getUserRolesTable());
  });

  it('should show users and attributes in table', () => {
    exist(benutzerPage.getUserEntry('ariane'));
    benutzerPage.stringExistsInUserEntry(role1, 'dorothea');
    benutzerPage.stringExistsInUserEntry(orga1, 'ludwig');
    benutzerPage.stringExistsInUserEntry(role1, 'zelda');
    benutzerPage.stringExistsInUserEntry(role2, 'zelda');
    benutzerPage.stringExistsInUserEntry(orga2, 'zelda');
    benutzerPage.stringExistsInUserEntry(orga3, 'zelda');
    benutzerPage.stringExistsInUserEntry(orga_none, 'adelheit');
    benutzerPage.stringExistsInUserEntry(mail1, 'peter');
    benutzerPage.stringExistsInUserEntry(role3, 'peter');
  });

  it('should show single user screen on click', () => {
    benutzerPage.addUser();

    exist(benutzerPage.getVornameInput());
    exist(benutzerPage.getNachnameInput());
    exist(benutzerPage.getBenutzernameInput());
    exist(benutzerPage.getMailInput());

    notBeChecked(benutzerPage.getAdminCheckbox());
    notBeChecked(benutzerPage.getLoeschenCheckbox());
    notBeChecked(benutzerPage.getUserCheckbox());
    notBeChecked(benutzerPage.getPostCheckbox());
  });

  it('should activate loeschen checkbox and deactivate the other two checkboxes', () => {
    benutzerPage.clickLoeschenCheckbox();
    beChecked(benutzerPage.getLoeschenCheckbox());
    notBeEnabled(benutzerPage.getUserCheckbox());
    notBeEnabled(benutzerPage.getPostCheckbox());

    benutzerPage.clickLoeschenCheckbox();
    notBeChecked(benutzerPage.getLoeschenCheckbox());
    beEnabled(benutzerPage.getUserCheckbox());
    beEnabled(benutzerPage.getPostCheckbox());
  });

  it('should additionally activate and deactivate admin checkbox', () => {
    benutzerPage.clickLoeschenCheckbox();
    benutzerPage.clickAdminCheckbox();
    beChecked(benutzerPage.getLoeschenCheckbox());
    beChecked(benutzerPage.getAdminCheckbox());

    benutzerPage.clickAdminCheckbox();
    notBeChecked(benutzerPage.getAdminCheckbox());
  });

  it('should activate user checkbox and deactivate the other two checkboxes', () => {
    benutzerPage.clickLoeschenCheckbox();
    benutzerPage.clickUserCheckbox();
    beChecked(benutzerPage.getUserCheckbox());
    notBeEnabled(benutzerPage.getLoeschenCheckbox());
    notBeEnabled(benutzerPage.getPostCheckbox());

    benutzerPage.clickUserCheckbox();
    notBeChecked(benutzerPage.getUserCheckbox());
    beEnabled(benutzerPage.getLoeschenCheckbox());
    beEnabled(benutzerPage.getPostCheckbox());
  });

  it('should activate post checkbox and deactivate the other two checkboxes', () => {
    benutzerPage.clickPostCheckbox();
    beChecked(benutzerPage.getPostCheckbox());
    notBeEnabled(benutzerPage.getLoeschenCheckbox());
    notBeEnabled(benutzerPage.getUserCheckbox());

    benutzerPage.clickPostCheckbox();
    notBeChecked(benutzerPage.getPostCheckbox());
    beEnabled(benutzerPage.getLoeschenCheckbox());
    beEnabled(benutzerPage.getUserCheckbox());
  });
});
