import { ApiRootLinkRel, ApiRootResource, ApiRootService } from '@alfa-client/api-root-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Params, Router } from '@angular/router';
import { AuthenticationService } from 'libs/authentication/src/lib/authentication.service';
import { Observable } from 'rxjs';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit {
  readonly title = 'admin';

  public apiRootStateResource$: Observable<StateResource<ApiRootResource>>;

  constructor(
    public authenticationService: AuthenticationService,
    private apiRootService: ApiRootService,
    private router: Router,
    private route: ActivatedRoute,
  ) {}

  ngOnInit(): void {
    this.authenticationService.login().then(() => this.doAfterLoggedIn());
  }

  doAfterLoggedIn(): void {
    this.apiRootStateResource$ = this.apiRootService.getApiRoot();
    this.forwardWithoutAuthenticationParams();
  }

  forwardWithoutAuthenticationParams() {
    const queryParams = this.getQueryParamsWithoutAuthentication();
    this.router.navigate([], { queryParams });
  }

  private getQueryParamsWithoutAuthentication(): Params {
    const { iss, state, session_state, code, ...queryParams } = this.route.snapshot.queryParams;
    return queryParams;
  }

  protected readonly ApiRootLinkRel = ApiRootLinkRel;
}
