/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UserProfileE2EComponent } from '../../../components/user-profile/user-profile.component.e2e';
import { VorgangListItemE2EComponent } from '../../../components/vorgang/vorgang-item.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { UserE2E } from '../../../model/user';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear, waitforSpinnerToAppear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections, wait } from '../../../support/cypress-helper';
import { enterWith, exist, haveText, notExist } from '../../../support/cypress.util';
import { getUserDorothea, getUserSabine, initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

describe('User Profile im ausgewählten Vorgang', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const userProfileContainer: UserProfileE2EComponent = vorgangPage.getVorgangDetailHeader().getUserContainer();

  const vorgang: VorgangE2E = createVorgang();

  const userSabine: UserE2E = getUserSabine();
  const userDorothea: UserE2E = getUserDorothea();

  before(() => {
    initVorgang(vorgang);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('icon in list', () => {
    it('should have icon', () => {
      const item: VorgangListItemE2EComponent = vorgangList.getListItem(vorgang.name);

      exist(item.getUserProfile().getRoot());
    });
  });

  describe('icon in vorgang detail', () => {
    it('open vorgang detail', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show icon', () => {
      exist(userProfileContainer.getIconContainer().getRoot());
    });

    it('should show formular on icon click', () => {
      userProfileContainer.getIconContainer().getRoot().click();

      exist(userProfileContainer.getSearchContainer().getRoot());
    });

    describe('fill formular', () => {
      it('should show error on enter wrong input', () => {
        enterWith(userProfileContainer.getSearchContainer().getInput(), 'Clark Kent');
        cy.wait(1000);

        exist(userProfileContainer.getSearchContainer().getError());
        exist(userProfileContainer.getIconContainer().getUnassignedIcon());
      });

      it('should show error on enter empty field', () => {
        enterWith(userProfileContainer.getSearchContainer().getInput(), 'Clark Kent', 500);
        cy.wait(1000);

        exist(userProfileContainer.getSearchContainer().getError());
        exist(userProfileContainer.getIconContainer().getUnassignedIcon());
      });

      describe('on valid formular', () => {
        it('should hide formular', () => {
          userProfileContainer.getSearchContainer().getInput().clear().type(userSabine.name);
          exist(userProfileContainer.getSearchContainer().getSearchOption(userSabine.dataTestId));
          userProfileContainer.getSearchContainer().getSearchOption(userSabine.dataTestId).click();

          notExist(userProfileContainer.getSearchContainer().getError());
        });

        it('should show assign icon', { defaultCommandTimeout: 30000 }, () => {
          waitForSpinnerToDisappear();
          exist(userProfileContainer.getIconContainer().getAssignedIcon());
          haveText(userProfileContainer.getIconContainer().getAssignedIcon(), userSabine.initials);
        });
      });
    });

    describe('navigate to list', () => {
      it('should show assigned icon', () => {
        vorgangPage.getSubnavigation().getBackButton().click();

        exist(vorgangList.getRoot());
        waitForSpinnerToDisappear();

        exist(vorgangList.getListItem(vorgang.name).getUserProfile().getRoot());
        haveText(
          vorgangList.getListItem(vorgang.name).getUserProfile().getIconContainer().getAssignedIcon(),
          userSabine.initials,
        );
      });
    });

    describe('open vorgang detail', () => {
      it('open vorgang detail', () => {
        vorgangList.getListItem(vorgang.name).getRoot().click();
        waitForSpinnerToDisappear();

        exist(vorgangPage.getVorgangDetailHeader().getRoot());
      });
    });

    describe('click on subnavigation icon button', () => {
      it('should show formular on icon click', () => {
        vorgangPage.getSubnavigation().getAssignUserProfileIconButton().click();

        exist(userProfileContainer.getSearchContainer().getRoot());
      });

      describe('on valid formular', () => {
        it('should hide formular', () => {
          userProfileContainer.getSearchContainer().getInput().clear().type(userDorothea.name);
          exist(userProfileContainer.getSearchContainer().getSearchOption(userDorothea.dataTestId));
          userProfileContainer.getSearchContainer().getSearchOption(userDorothea.dataTestId).click();

          waitforSpinnerToAppear();
          waitForSpinnerToDisappear();

          notExist(userProfileContainer.getSearchContainer().getError());
        });

        it('should show assign icon', { defaultCommandTimeout: 30000 }, () => {
          waitForSpinnerToDisappear();
          // WORKAROUND Hier muss aufs Verarbeiten im Backend gewartet werden - oder: wait(500)
          wait(500);
          exist(userProfileContainer.getIconContainer().getAssignedIcon());
          haveText(userProfileContainer.getIconContainer().getAssignedIcon(), userDorothea.initials);
        });
      });
    });

    describe('navigate to list', () => {
      it('should show assigned icon', () => {
        vorgangPage.getSubnavigation().getBackButton().click();

        exist(vorgangList.getRoot());
        waitForSpinnerToDisappear();

        exist(vorgangList.getListItem(vorgang.name).getUserProfile().getRoot());
        haveText(
          vorgangList.getListItem(vorgang.name).getUserProfile().getIconContainer().getAssignedIcon(),
          userDorothea.initials,
        );
      });
    });
  });
});
