/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangBescheidWizardE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheid-wizard.e2e.component';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import {
  VorgangMoreMenuE2EComponent,
  VorgangMoreMenuExportierenItemE2EComponent,
} from 'apps/alfa-e2e/src/components/vorgang/vorgang-more-menu.e2e.components';
import { CommandE2E, CommandOrderE2E } from 'apps/alfa-e2e/src/model/command';
import { KommentarE2E } from 'apps/alfa-e2e/src/model/kommentar';
import { VorgangAttachedItemE2E } from 'apps/alfa-e2e/src/model/vorgang-attached-item';
import { buildCommand, initCommands } from 'apps/alfa-e2e/src/support/command-util';
import { TEST_FILE_BESCHEID_VALID } from 'apps/alfa-e2e/src/support/data.util';
import { uploadFile } from 'apps/alfa-e2e/src/support/file-upload';
import {
  createKommentar,
  createKommentarAttachedItem,
} from 'apps/alfa-e2e/src/support/kommentar.util';
import { initVorgangAttachedItem } from 'apps/alfa-e2e/src/support/vorgang-attached-item-util';
import { objectIds } from 'apps/alfa-e2e/src/support/vorgang-util';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import {
  deleteDownloadFolder,
  dropCollections,
  getDownloadFileAt,
  getDownloadFiles,
  unzipDownloadFile,
} from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { parseXml } from '../../../support/tech.util';
import { getUserDorotheaId, initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('check xDomea contents', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangMoreMenu: VorgangMoreMenuE2EComponent = vorgangPage.getMoreMenu();
  const menuItem: VorgangMoreMenuExportierenItemE2EComponent = vorgangMoreMenu.getExportierenItem();

  const xdomeaNamespace: string = 'urn:xoev-de:xdomea:schema:3.0.0';
  //const ns3Namespace: string = 'urn:ozgcloud-de:xdomea:schema:1.0.0';

  const kommentarText: string = 'Kommentar zum Export';
  let xdomeaFile: string;

  const vorgangExportieren: VorgangE2E = {
    ...createVorgang(),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
    name: 'DoExportieren',
  };

  const assignUserCommand: CommandE2E = {
    ...buildCommand(
      CommandOrderE2E.ASSIGN_USER,
      vorgangExportieren._id.$oid,
      vorgangExportieren._id.$oid,
    ),
    bodyObject: { assignedTo: getUserDorotheaId() },
    finishedAt: { $date: '2024-06-20T07:25:30.000Z' },
  };

  const changeStatusCommand: CommandE2E = {
    ...buildCommand(
      CommandOrderE2E.VORGANG_WIEDEREROEFFNEN,
      vorgangExportieren._id.$oid,

      vorgangExportieren._id.$oid,
    ),
    order: CommandOrderE2E.VORGANG_WIEDEREROEFFNEN,
    finishedAt: { $date: '2024-06-18T07:25:30.000Z' },
  };

  const setAktenzeichenCommand: CommandE2E = {
    ...buildCommand(
      CommandOrderE2E.SET_AKTENZEICHEN,
      vorgangExportieren._id.$oid,
      vorgangExportieren._id.$oid,
    ),
    order: CommandOrderE2E.SET_AKTENZEICHEN,
    bodyObject: { aktenzeichen: 'AKT_ENZ_EIC_HEN1' },
    finishedAt: { $date: '2024-06-19T07:25:30.000Z' },
  };

  const kommentar: KommentarE2E = {
    ...createKommentar(),
    text: kommentarText,
  };
  const kommentarAttachedItem: VorgangAttachedItemE2E = {
    ...createKommentarAttachedItem(objectIds[1], vorgangExportieren._id.$oid),
    item: kommentar,
  };

  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();

  const bescheidWizard: VorgangBescheidWizardE2EComponent = vorgangPage.getBescheidWizard();

  before(() => {
    initVorgang(vorgangExportieren);
    initCommands([assignUserCommand, changeStatusCommand, setAktenzeichenCommand]);
    initVorgangAttachedItem([kommentarAttachedItem]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('create Bescheid, export and unzip file', () => {
    it('should create a Bescheid', () => {
      vorgangList.getListItem(vorgangExportieren.name).getRoot().click();
      vorgangFormularButtons.getBescheidenButton().click();
      bescheidWizard.weiter();
      uploadFile(bescheidWizard.getUploadBescheidButton(), TEST_FILE_BESCHEID_VALID);
      notExist(bescheidWizard.getBescheidUploadSpinner());
      bescheidWizard.weiter();
      bescheidWizard.getSendButton().click();
      waitForSpinnerToDisappear();
      vorgangFormularButtons.getAbschliessenButton().click();
    });

    it('should download and unzip the xdomea file', () => {
      waitForSpinnerToDisappear();
      openXdomeaFile();
    });

    it('should read the xdomea content', () => {
      readXdomeaFile();
    });
  });

  describe('check contents of xdomea file', () => {
    it('should contain entries for AnwendungsspezifischeErweiterung', () => {
      const xmlMap: Map<XmlTypePosition, string> = new Map<XmlTypePosition, string>();
      xmlMap.set({ type: 'Kennung', position: 0 }, 'IDOZGCloud1234567');
      xmlMap.set({ type: 'Name', position: 6 }, 'Anwendungsspezifische Erweiterung OZGCloud Basis');
      xmlMap.set({ type: 'Wert', position: 0 }, '2021-02-11T17:17:28Z');
      xmlMap.set({ type: 'Wert', position: 1 }, 'Testermann');
      xmlMap.set({ type: 'Wert', position: 2 }, 'Max');
      xmlMap.set({ type: 'Wert', position: 3 }, '1995-03-21');
      xmlMap.set({ type: 'Wert', position: 4 }, '66X00');
      compareXmlEntry(xdomeaFile, xmlMap);
    });

    it('should contain change of Vorgang Status', () => {
      const xmlMap: Map<XmlTypePosition, string> = new Map<XmlTypePosition, string>();
      xmlMap.set({ type: 'MetadatumNeuerWert', position: 0 }, 'In Bearbeitung');
      xmlMap.set({ type: 'DatumUhrzeit', position: 0 }, '2024-06-18T07:25:30.000Z');
      xmlMap.set({ type: 'Aktion', position: 0 }, CommandOrderE2E.VORGANG_WIEDEREROEFFNEN);
      compareXmlEntry(xdomeaFile, xmlMap);
    });

    it('should contain change of Aktenzeichen', () => {
      const xmlMap: Map<XmlTypePosition, string> = new Map<XmlTypePosition, string>();
      xmlMap.set({ type: 'MetadatumNeuerWert', position: 3 }, 'AKT_ENZ_EIC_HEN1');
      xmlMap.set({ type: 'DatumUhrzeit', position: 3 }, '2024-06-19T07:25:30.000Z');
      xmlMap.set({ type: 'Aktion', position: 3 }, 'SET_AKTENZEICHEN');
      compareXmlEntry(xdomeaFile, xmlMap);
    });

    it('should contain change of Bearbeiter', () => {
      const xmlMap: Map<XmlTypePosition, string> = new Map<XmlTypePosition, string>();
      xmlMap.set({ type: 'MetadatumNeuerWert', position: 4 }, 'Dorothea Doe; 9030229');
      xmlMap.set({ type: 'DatumUhrzeit', position: 4 }, '2024-06-20T07:25:30.000Z');
      xmlMap.set({ type: 'Aktion', position: 4 }, 'ASSIGN_USER');
      compareXmlEntry(xdomeaFile, xmlMap);
    });

    it('should contain creation of Kommentar', () => {
      const xmlMap: Map<XmlTypePosition, string> = new Map<XmlTypePosition, string>();
      xmlMap.set({ type: 'Typ', position: 0 }, 'Notiz');
      xmlMap.set({ type: 'Akteur', position: 5 }, 'Sabine Sach; 9030229');
      xmlMap.set({ type: 'DatumUhrzeit', position: 5 }, '2024-01-10T12:57:35.000Z');
      xmlMap.set({ type: 'Aktion', position: 5 }, 'CREATE_KOMMENTAR');
      compareXmlEntry(xdomeaFile, xmlMap);
    });

    it('should contain Bescheid info', () => {
      const xmlMap: Map<XmlTypePosition, string> = new Map<XmlTypePosition, string>();
      xmlMap.set({ type: 'Typ', position: 2 }, 'Bescheid');
      xmlMap.set({ type: 'Akteur', position: 7 }, 'Sabine Sach; 9030229');
      xmlMap.set({ type: 'Aktion', position: 7 }, 'Bescheid an Antragsteller gesendet');
      compareXmlEntry(xdomeaFile, xmlMap);
    });
  });

  function openXdomeaFile(): void {
    vorgangMoreMenu.openMenu();
    deleteDownloadFolder().then(() => {
      menuItem.getButton().click({ force: true });
      waitForSpinnerToDisappear();
      cy.wait(1000);
      getDownloadFiles().then((files) => {
        unzipDownloadFile(files[0]);
      });
    });
  }

  //TODO: Funktionen für NS3-Tag schreiben / anpassen
  function getXdomeaNamespace(content: string, tagName: string, position: number): string {
    return getXMLFromTagAtPosition(xdomeaNamespace, content, tagName, position);
  }

  function getXMLFromTagAtPosition(
    namespaceURI: string,
    content: string,
    tagName: string,
    position: number,
  ) {
    const xmlDoc = parseXml(content);
    return xmlDoc.getElementsByTagNameNS(namespaceURI, tagName)[position].textContent;
  }

  function readXdomeaFile(): void {
    findFirstDownloadFileWithString('Abgabe.0401.xml').then((xmlFile) => {
      getDownloadFileAt(xmlFile).then((content) => {
        xdomeaFile = content;
      });
    });
  }

  function findFirstDownloadFileWithString(searchString: string): Cypress.Chainable<number> {
    return getDownloadFiles().then((files) => {
      const targetFiles = files.filter((file) => {
        return file.toLowerCase().includes(searchString.toLowerCase());
      });

      if (targetFiles.length === 0) {
        throw new Error(`No files containing "${searchString}" found.`);
      }

      const targetFileName = targetFiles[0];
      const position = files.indexOf(targetFileName);
      return position;
    });
  }

  function compareXmlEntry(xDomeaFile: string, xmlMap: Map<XmlTypePosition, string>) {
    xmlMap.forEach((expectedValue, xmlTypePosition) => {
      expect(
        getXdomeaNamespace(xDomeaFile, xmlTypePosition.type, xmlTypePosition.position),
      ).to.equal(expectedValue);
    });
  }

  interface XmlTypePosition {
    type: string;
    position: number;
  }
});
