/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { isNotEmpty } from '@alfa-client/tech-shared';
import { isEmpty } from 'lodash-es';
import { UserE2E } from '../model/user';
import { UsermanagerUserE2E } from '../model/usermanager';
import { login } from './cypress-helper';

const sabineFixture: UserE2E = require('../fixtures/user-main/user_sabine.json');
const dorotheaFixture: UserE2E = require('../fixtures/user-main/user_dorothea.json');
const zeldaFixture: UserE2E = require('../fixtures/user-main/user_zelda.json');
const arianeFixture: UserE2E = require('../fixtures/user-main/user_ariane.json');

const userManagerSabineFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_sabine.json');
const userManagerPeterFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_peter.json');
const userManagerEmilFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_emil.json');
const userManagerDorotheaFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_dorothea.json');
const userManagerZeldaFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_zelda.json');
const userManagerArianeFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_ariane.json');
const userIds: { [username: string]: string } = require('../fixtures/usermanager/user-ids.json');

type Username = 'sabine' | 'peter' | 'emil' | 'adelheit' | 'ludwig' | 'richard' | 'zonk' | 'dorothea';

export function getUserId(username: Username) {
  if (isEmpty(userIds)) {
    throw new Error('user ids from fixtures/usermanager/user-ids.json empty');
  }
  if (isEmpty(userIds[username])) {
    throw new Error(`username ${username} not found in fixtures/usermanager/user-ids.json`);
  }
  return userIds[username];
}

export function areUsersSynced(): boolean {
  return isNotEmpty(userIds);
}

export function initUsermanagerUsers() {
  // initUsermanagerData([
  //   getUserManagerUserSabine(),
  //   getUserManagerUserPeter(),
  //   getUserManagerUserEmil(),
  //   getUserManagerUserDorothea(),
  //   getUserManagerUserZelda(),
  //   getUserManagerUserAriane(),
  // ]);
}

export function getUserSabine(): UserE2E {
  return sabineFixture;
}

export function getUserDorothea(): UserE2E {
  return dorotheaFixture;
}

export function getUserZelda(): UserE2E {
  return zeldaFixture;
}

export function getUserAriane(): UserE2E {
  return arianeFixture;
}

export function getUserManagerUserSabine(): UsermanagerUserE2E {
  return userManagerSabineFixture;
}

export function getUserManagerUserPeter(): UsermanagerUserE2E {
  return userManagerPeterFixture;
}

export function getUserManagerUserEmil(): UsermanagerUserE2E {
  return userManagerEmilFixture;
}

export function getUserManagerUserDorothea(): UsermanagerUserE2E {
  return userManagerDorotheaFixture;
}

export function getUserManagerUserZelda(): UsermanagerUserE2E {
  return userManagerZeldaFixture;
}

export function getUserManagerUserAriane(): UsermanagerUserE2E {
  return userManagerArianeFixture;
}

export function getUserSabineId(): string {
  return getUserId('sabine');
}

export function getUserDorotheaId(): string {
  return getUserId('dorothea');
}

export enum DatabaseUser {
  EMIL = 'user-ea/user_emil.json',
  ADELHEIT = 'user-main/user_adelheit.json',
  LUDWIG = 'user-main/user_ludwig.json',
  PETER = 'user-main/user_peter.json',
  RICHARD = 'user-main/user_richard.json',
  SABINE = 'user-main/user_sabine.json',
  DOROTHEA = 'user-main/user_dorothea.json',
  ZELDA = 'user-main/user_zelda.json',
  ZONK = 'user-main/user_zonk.json',
  ARIANE = 'user-main/user_ariane.json',
}

export function loginAsEmil(): void {
  login(DatabaseUser.EMIL);
}

export function loginAsAdelheit(): void {
  login(DatabaseUser.ADELHEIT);
}

export function loginAsPeter(): void {
  login(DatabaseUser.PETER);
}

export function loginAsRichard(): void {
  login(DatabaseUser.RICHARD);
}

export function loginAsZelda(): void {
  login(DatabaseUser.ZELDA);
}

export function loginAsSabine(): void {
  login(DatabaseUser.SABINE);
}

export function loginAsZonk(): void {
  login(DatabaseUser.ZONK);
}

export function loginAsLudwig(): void {
  login(DatabaseUser.LUDWIG);
}

export function loginAsAriane(): void {
  login(DatabaseUser.ARIANE);
}

export enum UserRoleE2E {
  EINHEITLICHER_ANSPRECHPARTNER = 'EINHEITLICHER_ANSPRECHPARTNER',
  VERWALTUNG_LOESCHEN = 'VERWALTUNG_LOESCHEN',
  VERWALTUNG_POSTSTELLE = 'VERWALTUNG_POSTSTELLE',
  VERWALTUNG_USER = 'VERWALTUNG_USER',
  ADMIN_ADMIN = 'ADMIN_ADMIN',
}

//TODO Cleanup
export function loginByUi(user: UserE2E): void {
  cy.visit('')
    .get('#kc-login')
    .should('exist')
    .get('#username')
    .type(user.name)
    .get('#password')
    .type(user.password)
    .get('#kc-login')
    .click();
}
