import { createEmptyStateResource, createStateResource, ProblemDetail } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, existsAsHtmlElement, Mock, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createInvalidParam, createProblemDetail } from 'libs/tech-shared/test/error';
import { singleCold } from 'libs/tech-shared/test/marbles';
import { MockComponent } from 'ng-mocks';
import { EMPTY } from 'rxjs';
import { createPostfachResource } from '../../../../../test/postfach/postfach';
import { PostfachResource } from '../../postfach.model';
import { PostfachService } from '../../postfach.service';
import { PostfachFormComponent } from './postfach-form.component';
import { PostfachSignaturComponent } from './postfach-signatur/postfach-signatur.component';
import { PostfachFormService } from './postfach.formservice';

describe('PostfachFormComponent', () => {
  let component: PostfachFormComponent;
  let fixture: ComponentFixture<PostfachFormComponent>;
  let formService: PostfachFormService;

  const postfachService: Mock<PostfachService> = mock(PostfachService);

  const saveButtonSelector: string = getDataTestIdOf('save-button');
  const invalidMessageSpan: string = getDataTestIdOf('invalid-empty-message-span');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [PostfachFormComponent, MockComponent(PostfachSignaturComponent), MockComponent(ButtonWithSpinnerComponent)],
      imports: [ReactiveFormsModule, FormsModule],
      providers: [
        {
          provide: PostfachService,
          useValue: postfachService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PostfachFormComponent);
    component = fixture.componentInstance;
    formService = component.formService;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('set postfachStateResource', () => {
    it('should return resource', () => {
      component.updatePostfachResource = jest.fn();

      const postfachResource: PostfachResource = createPostfachResource();
      component.postfachStateResource = createStateResource(postfachResource);

      expect(component.updatePostfachResource).toHaveBeenCalledWith(postfachResource);
    });
  });

  describe('updatePostfachResource', () => {
    it('should patch form with input postfach resource', () => {
      const patchFn = jest.spyOn(formService, 'patch');
      const postfachResource: PostfachResource = createPostfachResource();

      component.updatePostfachResource(postfachResource);

      expect(patchFn).toHaveBeenCalledWith(postfachResource.settingBody);
    });
  });

  // describe('Absender section', () => {
  //   const fields: string[][] = [
  //     [PostfachFormService.NAME_FIELD, 'Name', 'absender-name'],
  //     [PostfachFormService.ANSCHRIFT_FIELD, 'Anschrift', 'absender-anschrift'],
  //     [PostfachFormService.DIENST_FIELD, 'Dienst', 'absender-dienst'],
  //     [PostfachFormService.MANDANT_FIELD, 'Mandant', 'absender-mandant'],
  //     [
  //       PostfachFormService.GEMEINDESCHLUESSEL_FIELD,
  //       'Gemeindeschlüssel',
  //       'absender-gemeindeschluessel',
  //     ],
  //   ];
  //
  //   test.each(fields)(
  //     'should bind form for text-field "%s"',
  //     async (fieldName, text, dataTestId) => {
  //       const fieldValue = `some text-field ${text}`;
  //       const absenderGroup = formService.form.get(PostfachFormService.ASBSENDER_GROUP);
  //       const formControl = absenderGroup.get(fieldName);
  //
  //       const textFieldComponent = getDebugElementFromFixtureByCss(
  //         fixture,
  //         getDataTestIdOf(dataTestId),
  //       );
  //       ngMocks.change(textFieldComponent, fieldValue);
  //       expect(formControl.value).toBe(fieldValue);
  //     },
  //   );
  //
  //   test.each(fields)(
  //     'should have label for field "%s" with name "%s"',
  //     (fieldName, text, dataTestId) => {
  //       const textFieldElement = getElementFromFixture(fixture, getDataTestIdOf(dataTestId));
  //       expect(textFieldElement.getAttribute('ng-reflect-label')).toBe(text);
  //     },
  //   );
  // });

  describe('save button', () => {
    it('should call submit on click', () => {
      const submitFn: jest.SpyInstance = jest.spyOn(formService, 'submit');
      postfachService.save.mockReturnValue(EMPTY);

      dispatchEventFromFixture(fixture, saveButtonSelector, 'clickEmitter');

      expect(submitFn).toHaveBeenCalled();
    });

    // it.each([true, false])('should use submit progress "%s"', (progress) => {
    //   component.submitInProgress$ = of(progress);
    //
    //   fixture.detectChanges();
    //
    //   const saveButtonComponent: ButtonWithSpinnerComponent = getDebugElementFromFixtureByCss(
    //     fixture,
    //     saveButtonSelector,
    //   ).componentInstance;
    //   expect(saveButtonComponent.loading).toBe(progress);
    // });
  });

  describe('invalid message', () => {
    it('should show if form is invalid', () => {
      formService.setErrorByProblemDetail(createProblemDetailForAbsenderName());

      fixture.detectChanges();

      existsAsHtmlElement(fixture, invalidMessageSpan);
    });

    function createProblemDetailForAbsenderName(): ProblemDetail {
      return {
        ...createProblemDetail(),
        invalidParams: [{ ...createInvalidParam(), name: 'settingBody.absender.name' }],
      };
    }

    it('should not show if form valid', () => {
      expect(formService.form.invalid).toBeFalsy();

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, invalidMessageSpan);
    });
  });

  describe('submit', () => {
    it('should assign submit in progress', () => {
      const stateResource = createEmptyStateResource();
      formService.submit = jest.fn().mockReturnValue(singleCold(stateResource));

      component.submit();

      expect(component.submitInProgress$).toBeObservable(singleCold(stateResource));
    });
  });
});
