import { getElementFromFixture, mock, Mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { TextareaEditorComponent } from '@ods/component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { PostfachService } from '../../../postfach.service';
import { PostfachFormComponent } from '../postfach-form.component';
import { PostfachFormService } from '../postfach.formservice';
import { PostfachSignaturComponent } from './postfach-signatur.component';

describe('PostfachSignaturComponent', () => {
  let component: PostfachSignaturComponent;
  let fixture: ComponentFixture<PostfachSignaturComponent>;

  let formService: Mock<PostfachFormService>;
  let postfachService: Mock<PostfachService>;
  const signaturTextarea = getDataTestIdOf('signatur-text');

  beforeEach(async () => {
    formService = mock(PostfachFormService);
    postfachService = mock(PostfachService);
    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule],
      declarations: [PostfachFormComponent, MockComponent(TextareaEditorComponent)],
      providers: [
        {
          provide: PostfachFormService,
          useValue: formService,
        },
        {
          provide: PostfachService,
          useValue: postfachService,
        },
      ],
    }).compileComponents();
    fixture = TestBed.createComponent(PostfachSignaturComponent);
    TestBed.inject(PostfachService);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('template', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    describe('ods-textarea-editor', () => {
      describe('input', () => {
        it('should set formControlName to Signatur Field', () => {
          const textAreaEditor = getElementFromFixture(fixture, signaturTextarea);

          expect(textAreaEditor.getAttribute('rows')).toEqual('6');
        });
      });
    });
  });
});
