import { AbstractFormService, EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { AbstractControl, FormControl, UntypedFormBuilder, UntypedFormGroup } from '@angular/forms';
import { Resource } from '@ngxp/rest';
import { Observable } from 'rxjs';

@Injectable()
export class UserAddFormService extends AbstractFormService {
  public static readonly VORNAME: string = 'vorname';
  public static readonly NACHNAME: string = 'nachname';
  public static readonly BENUTZERNAME: string = 'benutzername';
  public static readonly EMAIL: string = 'e-mail';

  public static readonly ROLLEN_GROUP: string = 'rollen';
  public static readonly ADMINISTRATION_GROUP: string = 'administration';
  public static readonly ADMIN: string = 'admin';
  public static readonly ALFA_GROUP: string = 'alfa';
  public static readonly LOESCHEN: string = 'loeschen';
  public static readonly USER: string = 'user';
  public static readonly POSTSTELLE: string = 'poststelle';

  public static readonly USER_ADD_PREFIX: string = 'userAdd';

  constructor(public formBuilder: UntypedFormBuilder) {
    super(formBuilder);
    this.initAlfaGroupLogic();
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [UserAddFormService.VORNAME]: new FormControl(EMPTY_STRING),
      [UserAddFormService.NACHNAME]: new FormControl(EMPTY_STRING),
      [UserAddFormService.BENUTZERNAME]: new FormControl(EMPTY_STRING),
      [UserAddFormService.EMAIL]: new FormControl(EMPTY_STRING),
      [UserAddFormService.ROLLEN_GROUP]: this.formBuilder.group({
        [UserAddFormService.ADMINISTRATION_GROUP]: this.formBuilder.group({
          [UserAddFormService.ADMIN]: new FormControl(false),
        }),
        [UserAddFormService.ALFA_GROUP]: this.formBuilder.group({
          [UserAddFormService.LOESCHEN]: new FormControl(false),
          [UserAddFormService.USER]: new FormControl(false),
          [UserAddFormService.POSTSTELLE]: new FormControl(false),
        }),
      }),
    });
  }

  protected initAlfaGroupLogic(): void {
    const alfaGroup: UntypedFormGroup = this.getAlfaGroup();
    alfaGroup.valueChanges.subscribe(() => {
      this.handleAlfaGroupChange(alfaGroup);
    });
  }

  private getAlfaGroup(): UntypedFormGroup {
    return <UntypedFormGroup>this.form.get(UserAddFormService.ROLLEN_GROUP).get(UserAddFormService.ALFA_GROUP);
  }

  handleAlfaGroupChange(group: UntypedFormGroup): void {
    const anyChecked: boolean = this.isAnyChecked(group);
    if (anyChecked) {
      this.disableUncheckedCheckboxes(group);
    } else {
      this.enableAllCheckboxes(group);
    }
  }

  isAnyChecked(group: UntypedFormGroup): boolean {
    return Object.keys(group.controls).some((key) => group.controls[key].value);
  }

  disableUncheckedCheckboxes(alfaGroup: UntypedFormGroup): void {
    for (const control of Object.values<AbstractControl>(alfaGroup.controls)) {
      if (control.value === false) control.disable({ emitEvent: false });
    }
  }

  enableAllCheckboxes(group: UntypedFormGroup): void {
    for (const control of Object.values<AbstractControl>(group.controls)) {
      control.enable({ emitEvent: false });
    }
  }

  protected doSubmit(): Observable<StateResource<Resource>> {
    throw new Error('Method not implemented.');
  }

  protected getPathPrefix(): string {
    return UserAddFormService.USER_ADD_PREFIX;
  }
}
