import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { VorgangResource, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { Component, Input, OnChanges, SimpleChanges } from '@angular/core';
import { getUrl, hasLink } from '@ngxp/rest';
import { has } from 'lodash-es';
import { Observable, tap } from 'rxjs';

@Component({
  selector: 'alfa-bescheid-in-vorgang-container',
  templateUrl: './bescheid-in-vorgang-container.component.html',
  styles: [],
})
export class BescheidInVorgangContainerComponent implements OnChanges {
  @Input() vorgang: VorgangResource;

  public bescheidStateResource$: Observable<StateResource<BescheidResource>>;
  public bescheidDocumentFile$: Observable<StateResource<BinaryFileResource>>;

  get hasBescheidDraft(): boolean {
    return hasLink(this.vorgang, VorgangWithEingangLinkRel.BESCHEID_DRAFT);
  }

  constructor(private bescheidService: BescheidService) {}

  // TODO Prüfen, warum nicht als Setter möglich. Hatte im ersten Versuch zu Laufzeitfehlern geführt.
  ngOnChanges(changes: SimpleChanges): void {
    if (!has(changes, 'vorgang')) {
      return;
    }

    this.bescheidStateResource$ = this.getBescheid();
    this.bescheidDocumentFile$ = this.bescheidService.getBescheidDocumentFile();
  }

  getBescheid(): Observable<StateResource<BescheidResource>> {
    return this.bescheidService.getBescheidDraft().pipe(
      tap((bescheidResource) => {
        if (hasLink(bescheidResource.resource, BescheidLinkRel.BESCHEID_DOCUMENT)) {
          this.bescheidService.loadBescheidDocumentByUri(
            getUrl(bescheidResource.resource, BescheidLinkRel.BESCHEID_DOCUMENT),
          );
        }
      }),
    );
  }
}
