import { BescheidService } from '@alfa-client/bescheid-shared';
import { Mock, existsAsHtmlElement, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';

import { CloseIconComponent, StampIconComponent } from '@ods/system';
import { createBescheidResource } from 'libs/bescheid-shared/src/test/bescheid';
import { FormatFullDatePipe } from 'libs/tech-shared/src/lib/pipe/format-full-date.pipe';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { of } from 'rxjs';
import { BeschiedenDateContainerComponent } from './beschieden-date-container.component';

registerLocaleData(localeDe);

describe('BeschiedenDateContainerComponent', () => {
  let component: BeschiedenDateContainerComponent;
  let fixture: ComponentFixture<BeschiedenDateContainerComponent>;

  const bescheidService: Mock<BescheidService> = mock(BescheidService);

  const bewilligtIcon: string = getDataTestIdOf('bewilligt-icon');
  const abgelehntIcon: string = getDataTestIdOf('abgelehnt-icon');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BeschiedenDateContainerComponent,
        FormatFullDatePipe,
        MockComponent(StampIconComponent),
        MockComponent(CloseIconComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BeschiedenDateContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOninit', () => {
    it('should call service', () => {
      component.ngOnInit();

      expect(bescheidService.getLastBescheid).toHaveBeenCalled();
    });
  });

  describe('on bewilligt', () => {
    describe('true', () => {
      beforeEach(() => {
        component.bescheid$ = of({ ...createBescheidResource(), bewilligt: true });
      });

      it('should show bewilligt icon', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, bewilligtIcon);
      });

      it('should hide abgelehnt icon', () => {
        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, abgelehntIcon);
      });
    });

    describe('false', () => {
      beforeEach(() => {
        component.bescheid$ = of({ ...createBescheidResource(), bewilligt: false });
      });

      it('should hide bewilligt icon', () => {
        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bewilligtIcon);
      });

      it('should show abgelehnt icon', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, abgelehntIcon);
      });
    });
  });
});
