import { ApiRootService } from '@alfa-client/api-root-shared';
import { BinaryFileResource, BinaryFileService } from '@alfa-client/binary-file-shared';
import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BinaryFile2ContainerComponent } from './binary-file2-container.component';
import { BinaryFile2Component } from './binary-file2/binary-file2.component';

describe('BinaryFile2ContainerComponent', () => {
  let component: BinaryFile2ContainerComponent;
  let fixture: ComponentFixture<BinaryFile2ContainerComponent>;

  const service = mock(BinaryFileService);
  const apiRootService = mock(ApiRootService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFile2ContainerComponent,
        ConvertForDataTestPipe,
        MockComponent(BinaryFile2Component),
      ],
      providers: [
        {
          provide: BinaryFileService,
          useValue: service,
        },
        {
          provide: ApiRootService,
          useValue: apiRootService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BinaryFile2ContainerComponent);
    component = fixture.componentInstance;
    component.file = createBinaryFileResource();

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('start download', () => {
    const file: BinaryFileResource = createBinaryFileResource();
    const fileNameDownloadPrefix: string = 'FileNamePrefixDummy';

    it('should call file service downloadFile', () => {
      component.downloadFileNamePrefix = fileNameDownloadPrefix;

      component.startDownload(file);

      expect(service.downloadFile).toHaveBeenCalledWith(file, fileNameDownloadPrefix);
    });
  });

  describe('get download token', () => {
    it('should call ApiRootService.getDownloadToken()', () => {
      apiRootService.getDownloadToken.mockReturnValue(of(''));
      component.getDownloadToken();

      expect(apiRootService.getDownloadToken).toHaveBeenCalled();
    });
  });
});
