import { ApiDownloadToken } from '@alfa-client/api-root-shared';
import {
  BinaryFileIcon,
  BinaryFileLinkRel,
  BinaryFileResource,
} from '@alfa-client/binary-file-shared';
import { StateResource, createEmptyStateResource } from '@alfa-client/tech-shared';
import { HttpParams } from '@angular/common/http';
import { Component, EventEmitter, Input, Output } from '@angular/core';
import { Resource, getUrl } from '@ngxp/rest';
import { isEmpty, isNil } from 'lodash-es';

@Component({
  selector: 'alfa-binary-file2',
  templateUrl: './binary-file2.component.html',
})
export class BinaryFile2Component {
  @Input() file: BinaryFileResource;
  @Input() stateResource: StateResource<Resource>;
  @Input() deletable: boolean = false;
  @Input() downloadToken: ApiDownloadToken = <ApiDownloadToken>{};

  readonly fileLinkRel = BinaryFileLinkRel;

  @Output() public startDownload: EventEmitter<BinaryFileResource> =
    new EventEmitter<BinaryFileResource>();
  @Output() public startDelete: EventEmitter<BinaryFileResource> =
    new EventEmitter<BinaryFileResource>();
  @Output() public getDownloadToken: EventEmitter<void> = new EventEmitter<void>();

  get isDisabled(): boolean {
    return this.getStateResource().loading;
  }

  get isLoading(): boolean {
    return this.getStateResource().loading;
  }

  getStateResource(): StateResource<Resource> {
    return isNil(this.stateResource) ? createEmptyStateResource<Resource>() : this.stateResource;
  }

  getIconType(type: string): string {
    return BinaryFileIcon[type] ?? 'file';
  }

  downloadFile(): void {
    this.startDownload.emit(this.file);
  }

  deleteFile(): void {
    this.startDelete.emit(this.file);
  }

  getNewDownloadToken(): void {
    this.getDownloadToken.emit();
  }

  dragStart(event: DragEvent): void {
    event.dataTransfer.setData('DownloadURL', this.createDownloadUrl());
  }

  dragEnd(event: DragEvent): void {
    if (this.isValidDrop(event)) {
      this.handleSpinnerForDragEnd();
    }
  }

  private isValidDrop(event: DragEvent): boolean {
    return event.dataTransfer.dropEffect !== 'none';
  }

  handleSpinnerForDragEnd(): void {
    this.stateResource = { ...this.stateResource, loading: true };
    setTimeout(() => (this.stateResource = { ...this.stateResource, loading: false }), 3000);
  }

  createDownloadUrl(): string {
    return `${this.getContentType()}:${this.file.name}:${this.createRemoteUrl()}`;
  }

  private getContentType(): string {
    return isEmpty(this.file.contentType) ? 'application/octet-stream' : this.file.contentType;
  }

  private createRemoteUrl(): string {
    let getParam = new HttpParams();
    getParam = getParam.set('token', this.downloadToken?.token);

    return getUrl(this.file, BinaryFileLinkRel.DOWNLOAD) + '?' + getParam.toString();
  }
}
