import { CollaborationListResource } from '@alfa-client/collaboration-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, getMockComponent, mock } from '@alfa-client/test-utils';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CollaborationService } from 'libs/collaboration-shared/src/lib/collaboration.service';
import { createCollaborationListResource } from 'libs/collaboration-shared/test/collaboration';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { CollaborationInVorgangContainerComponent } from './collaboration-in-vorgang-container.component';
import { CollaborationInVorgangComponent } from './collaboration-in-vorgang/collaboration-in-vorgang.component';

describe('CollaborationInVorgangContainerComponent', () => {
  let component: CollaborationInVorgangContainerComponent;
  let fixture: ComponentFixture<CollaborationInVorgangContainerComponent>;

  let service: Mock<CollaborationService>;

  const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(async () => {
    service = mock(CollaborationService);
    await TestBed.configureTestingModule({
      declarations: [CollaborationInVorgangContainerComponent, MockComponent(CollaborationInVorgangComponent)],
      providers: [
        {
          provide: CollaborationService,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(CollaborationInVorgangContainerComponent);
    component = fixture.componentInstance;
    component.vorgang = vorgang;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service to get collaboration state list resource', () => {
      component.ngOnInit();

      expect(service.getList).toHaveBeenCalled();
    });

    it('should call service to get organisationsEinheit anfrage button visbility', () => {
      component.ngOnInit();

      expect(service.showOrganisationsEinheitAnfrageButton).toHaveBeenCalled();
    });

    it('should call service to get externeFachstelle anfrage button visbility', () => {
      component.ngOnInit();

      expect(service.showExterneFachstelleAnfrageButton).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    describe('collaboration in vorgang component', () => {
      const collaborationListResource: CollaborationListResource = createCollaborationListResource();
      const collaborationStateListResource: StateResource<CollaborationListResource> =
        createStateResource(collaborationListResource);

      describe('should be called with', () => {
        it('vorgang', () => {
          component.vorgang = vorgang;

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().vorgang).toBe(vorgang);
        });

        it('collaboration state list resource', () => {
          component.collaborationStateListResource$ = of(collaborationStateListResource);

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().collaborationStateListResource).toBe(collaborationStateListResource);
        });

        it('showOrganisationsEinheitAnfrageButton', () => {
          component.showOrganisationsEinheitAnfrageButton$ = of(true);

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().showOrganisationsEinheitAnfrageButton).toBeTruthy();
        });

        it('showExterneFachstelleAnfrageButton', () => {
          component.showExterneFachstelleAnfrageButton$ = of(true);

          fixture.detectChanges();

          expect(getCollaborationInVorgangComponent().showExterneFachstelleAnfrageButton).toBeTruthy();
        });

        function getCollaborationInVorgangComponent(): CollaborationInVorgangComponent {
          return getMockComponent<CollaborationInVorgangComponent>(fixture, CollaborationInVorgangComponent);
        }
      });
    });
  });
});
