import { CollaborationListResource } from '@alfa-client/collaboration-shared';
import { HasLinkPipe } from '@alfa-client/tech-shared';
import {
  Mock,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getMockComponent,
  mock,
  notExistsAsHtmlElement,
  useFromMock,
} from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { SearchExterneFachstelleContainerComponent } from '@alfa-client/zustaendige-stelle';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder } from '@angular/forms';
import { ExternalUnitIconComponent } from '@ods/system';
import { CollaborationService } from 'libs/collaboration-shared/src/lib/collaboration.service';
import { createCollaborationListResource } from 'libs/collaboration-shared/test/collaboration';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { CollaborationRequestFormComponent } from '../../collaboration-request-form/collaboration-request-form.component';
import { CollaborationRequestFormService } from '../../collaboration-request-form/collaboration.request.formservice';
import { AnfrageErstellenButtonComponent } from '../anfrage-erstellen-button/anfrage-erstellen-button.component';
import { ExterneFachstelleContainerComponent } from './externe-fachstelle-container.component';

describe('ExterneFachstelleContainerComponent', () => {
  let component: ExterneFachstelleContainerComponent;
  let fixture: ComponentFixture<ExterneFachstelleContainerComponent>;

  const anfrageErstellenButton: string = getDataTestIdOf('externe-fachstelle-anfrage-erstellen-button');
  const collaborationRequestForm: string = getDataTestIdOf('collaboration-request-form');
  const searchExterneFachstelleContainer: string = getDataTestIdOf('search-externe-fachstelle-container');

  let formService: CollaborationRequestFormService;
  let service: Mock<CollaborationService>;

  const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
  const collaborationListResource: CollaborationListResource = createCollaborationListResource();

  beforeEach(async () => {
    formService = new CollaborationRequestFormService(new FormBuilder(), useFromMock(mock(CollaborationService)));
    service = mock(CollaborationService);
    TestBed.overrideComponent(CollaborationRequestFormComponent, {
      set: {
        providers: [
          {
            provide: CollaborationRequestFormService,
            useValue: formService,
          },
        ],
      },
    });
    await TestBed.configureTestingModule({
      declarations: [
        ExterneFachstelleContainerComponent,
        HasLinkPipe,
        MockComponent(SearchExterneFachstelleContainerComponent),
        MockComponent(CollaborationRequestFormComponent),
        MockComponent(AnfrageErstellenButtonComponent),
        MockComponent(ExternalUnitIconComponent),
      ],
      providers: [
        {
          provide: CollaborationRequestFormService,
          useValue: formService,
        },
        {
          provide: CollaborationService,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(ExterneFachstelleContainerComponent);
    component = fixture.componentInstance;
    component.vorgang = vorgang;
    component.collaborationListResource = collaborationListResource;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service to get form visibility', () => {
      component.ngOnInit();

      expect(service.isExterneFachstelleFormVisible).toHaveBeenCalled();
    });
  });

  describe('ngOnDestroy', () => {
    it('should call hideForm', () => {
      component.hideForm = jest.fn();

      component.ngOnDestroy();

      expect(component.hideForm).toHaveBeenCalled();
    });
  });

  describe('hideForm', () => {
    it('should call service to hide form', () => {
      component.hideForm();

      expect(service.hideExterneFachstelleForm).toHaveBeenCalled();
    });

    it.skip('FIXME should call formService to reset values', () => {
      component.hideForm();

      expect(formService.reset).toHaveBeenCalled();
    });
  });

  describe('showForm', () => {
    it('should call service', () => {
      component.showForm();

      expect(service.showExterneFachstelleForm).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    describe('on shown form', () => {
      beforeEach(() => {
        component.isFormVisible$ = of(true);
        fixture.detectChanges();
      });

      it('anfrage button should be hidden', () => {
        notExistsAsHtmlElement(fixture, anfrageErstellenButton);
      });

      describe('request form', () => {
        it('should be visible', () => {
          existsAsHtmlElement(fixture, collaborationRequestForm);
        });

        describe('component', () => {
          it('should be called with collaborationListResource', () => {
            const comp: CollaborationRequestFormComponent = getMockComponent<CollaborationRequestFormComponent>(
              fixture,
              CollaborationRequestFormComponent,
            );
            expect(comp.collaborationListResource).toBe(collaborationListResource);
          });

          it('should call hideForm on abbrechen output', () => {
            component.hideForm = jest.fn();

            dispatchEventFromFixture(fixture, collaborationRequestForm, 'abbrechen');

            expect(component.hideForm).toHaveBeenCalled();
          });
        });
      });

      describe('and search link exists', () => {
        beforeEach(() => {
          component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.SEARCH_EXTERNE_FACHSTELLE]);
          fixture.detectChanges();
        });

        it('search externe fachstelle component should be visible', () => {
          existsAsHtmlElement(fixture, searchExterneFachstelleContainer);
        });
      });

      describe('and search link is NOT exists', () => {
        beforeEach(() => {
          component.vorgang = createVorgangWithEingangResource();
          fixture.detectChanges();
        });
        it('search externe fachstelle component should be hidden', () => {
          notExistsAsHtmlElement(fixture, searchExterneFachstelleContainer);
        });
      });
    });

    describe('on hidden form', () => {
      beforeEach(() => {
        component.isFormVisible$ = of(false);
        fixture.detectChanges();
      });
      describe('anfrage erstellen button', () => {
        it('should be visible', () => {
          existsAsHtmlElement(fixture, anfrageErstellenButton);
        });

        it('should call showForm on click output', () => {
          component.showForm = jest.fn();

          dispatchEventFromFixture(fixture, anfrageErstellenButton, 'clickEmitter');

          expect(component.showForm).toHaveBeenCalled();
        });
      });
    });
  });
});
