import { getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { TextareaEditorComponent } from './textarea-editor.component';

describe('TextareaEditorComponent', () => {
  let component: TextareaEditorComponent;
  let fixture: ComponentFixture<TextareaEditorComponent>;

  const labelText: string = faker.word.noun();
  const textAreaTestId: string = getDataTestIdOf(labelText + '-textarea-editor');
  const errorId: string = getDataTestIdOf(labelText + '-textarea-editor-error');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [TextareaEditorComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(TextareaEditorComponent);
    component = fixture.componentInstance;
    component.label = labelText;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should have text area', () => {
    const element: HTMLElement = getElementFromFixture(fixture, textAreaTestId);

    expect(element).toBeInstanceOf(HTMLElement);
  });

  it('should have focus set to false by default', () => {
    expect(component.focus).toBeFalsy();
  });

  describe('errors', () => {
    it('should be hidden', () => {
      const element: HTMLElement = getElementFromFixture(fixture, errorId);

      expect(element).toBeEmptyDOMElement();
    });

    // TODO: Set error and test for 'should be visible
  });
});
