import { FormControl } from '@angular/forms';
import { argsToTemplate, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';
import { CheckboxComponent } from './checkbox.component';

const formControlWithError = new FormControl(false);
formControlWithError.setErrors({ error: 404 });

const meta: Meta<CheckboxComponent> = {
  title: 'Checkbox',
  component: CheckboxComponent,
  decorators: [
    moduleMetadata({
      imports: [CheckboxComponent],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<CheckboxComponent>;

export const Default: Story = {
  args: {
    label: 'Basic checkbox',
    inputId: '1',
    disabled: false,
    hasError: false,
  },
  argTypes: {
    label: { description: 'Checkbox label' },
    disabled: { description: 'Disabled state of checkbox' },
    inputId: { description: 'Id of checkbox input' },
    hasError: { description: 'Has checkbox the error state' },
    fieldControl: {
      description: 'Form control object',
      table: { type: { summary: 'FormControl' } },
    },
  },
};

export const Error: Story = {
  args: {
    label: 'Checkbox with error state',
    inputId: '2',
    hasError: true,
  },
};

export const Disabled: Story = {
  args: {
    label: 'Disabled checkbox',
    inputId: '3',
    disabled: true,
  },
};

export const MultiLineLabel: Story = {
  args: {
    label: 'Very long label that should break',
    inputId: '4',
  },
  render: (args) => ({
    props: args,
    template: `<div class="w-40"><ods-checkbox ${argsToTemplate(args)}/></div>`,
  }),
};
