/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIconModule } from '@angular/material/icon';
import { MAT_SNACK_BAR_DATA, MatSnackBarRef } from '@angular/material/snack-bar';
import { mock } from '@alfa-client/test-utils';
import { MockComponent } from 'ng-mocks';
import { SnackbarCloseButtonComponent } from '../snackbar-close-button/snackbar-close-button.component';
import { SnackBarData } from '../snackbar.model';
import { SnackbarErrorComponent } from './snackbar-error.component';

describe('SnackbarErrorComponent', () => {
  let component: SnackbarErrorComponent;
  let fixture: ComponentFixture<SnackbarErrorComponent>;

  const snackBarData: SnackBarData = { message: 'Message' };
  const snackBarRef = { ...mock(MatSnackBarRef), dissmis: jest.fn(), dismissWithAction: jest.fn() };

  beforeEach(async () => {
    TestBed.configureTestingModule({
      imports: [MatIconModule],
      declarations: [SnackbarErrorComponent, MockComponent(SnackbarCloseButtonComponent)],
      providers: [
        {
          provide: MAT_SNACK_BAR_DATA,
          useValue: { data: snackBarData },
        },
        {
          provide: MatSnackBarRef,
          useValue: snackBarRef,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(SnackbarErrorComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('close button', () => {
    it('should close the snackbar', () => {
      component.close();

      expect(snackBarRef.dismiss).toHaveBeenCalled();
    });
  });
});
