/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createEmptyStateResource } from '@alfa-client/tech-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatProgressBar } from '@angular/material/progress-bar';
import { ProgressBarComponent } from './progress-bar.component';

describe('ProgressBarComponent', () => {
  let component: ProgressBarComponent;
  let fixture: ComponentFixture<ProgressBarComponent>;

  const overlaySelector = '[data-test-class="progressbar-overlay"]';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ProgressBarComponent],
      imports: [MatProgressBar],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(ProgressBarComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('progressbar overlay', () => {
    it('should NOT be visible if NOT loading and NOT loaded', () => {
      component.stateResource = { ...createEmptyStateResource(false), loaded: false };
      fixture.detectChanges();

      const overlay = fixture.nativeElement.querySelector(overlaySelector);

      expect(overlay).not.toBeInstanceOf(HTMLElement);
    });

    it('should NOT be visible if NOT loading and already loaded', () => {
      component.stateResource = { ...createEmptyStateResource(false), loaded: true };
      fixture.detectChanges();

      const overlay = fixture.nativeElement.querySelector(overlaySelector);

      expect(overlay).not.toBeInstanceOf(HTMLElement);
    });

    it('should be visible if loading', () => {
      component.stateResource = createEmptyStateResource(true);
      fixture.detectChanges();

      const overlay = fixture.nativeElement.querySelector(overlaySelector);

      expect(overlay).toBeInstanceOf(HTMLElement);
    });
  });
});
