import { BescheidService, DocumentResource } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { hasStateResourceError, isNotNil, StateResource } from '@alfa-client/tech-shared';
import { Component, Input, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { Observable } from 'rxjs';
import { BescheidenFormService } from '../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-result-nachricht',
  templateUrl: './vorgang-detail-bescheiden-result-nachricht.component.html',
})
export class VorgangDetailBescheidenResultNachrichtComponent implements OnInit {
  @Input()
  set bescheidDocumentStateResource(stateResource: StateResource<DocumentResource>) {
    if (!stateResource.loading && !stateResource.error && isNotNil(stateResource.resource)) {
      if (isNotNil(this.form) && this.form.valid) {
        this.formService.patchNachricht(stateResource.resource);
      }
    }
  }

  @Input()
  set sendWithNachrichtCommandStateResource(stateResource: StateResource<CommandResource>) {
    this.resetFocus();
    if (isNotNil(stateResource) && hasStateResourceError(stateResource)) {
      if (this.formService.isBetreffInvalid()) {
        this.focusBetreff = true;
      } else if (this.formService.isNachrichtInvalid()) {
        this.focusNachricht = true;
      }
    }
  }

  resetFocus() {
    this.focusBetreff = false;
    this.focusNachricht = false;
  }

  empfaenger$: Observable<string>;
  form: FormGroup;

  public focusBetreff: boolean = false;
  public focusNachricht: boolean = false;

  readonly formServiceClass = BescheidenFormService;

  constructor(
    private readonly bescheidService: BescheidService,
    private readonly formService: BescheidenFormService,
  ) {}

  ngOnInit(): void {
    this.empfaenger$ = this.bescheidService.getEmpfaenger();
    this.form = this.formService.form;
  }
}
