import {
  BescheidLinkRel,
  BescheidResource,
  BescheidService,
  UploadFileInProgress,
} from '@alfa-client/bescheid-shared';
import { isNotNil, StateResource } from '@alfa-client/tech-shared';
import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { ResourceUri } from '@ngxp/rest';
import { isNull } from 'lodash-es';
import { Observable, of, Subscription } from 'rxjs';
import { BescheidenFormService } from '../../../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-dokument-hochladen',
  templateUrl: './vorgang-detail-bescheiden-dokument-hochladen.component.html',
  styles: [],
})
export class VorgangDetailBescheidenDokumentHochladenComponent implements OnInit, OnDestroy {
  @Input() bescheidDraftStateResource: StateResource<BescheidResource>;

  public uploadDocumentFileInProgress$: Observable<UploadFileInProgress> = of({ loading: false });

  public readonly formServiceClass = BescheidenFormService;
  public readonly bescheidLinkRel = BescheidLinkRel;

  private documentUriChangeSubscription: Subscription;

  constructor(
    public readonly formService: BescheidenFormService,
    private bescheidService: BescheidService,
  ) {}

  ngOnInit(): void {
    this.documentUriChangeSubscription = this.bescheidService
      .getDocumentUri()
      .subscribe((documentUri) => this.handleDocumentUriChanges(documentUri));
  }

  handleDocumentUriChanges(uri: ResourceUri): void {
    if (isNull(uri)) {
      this.formService.clearBescheidDocumentFile();
    } else {
      this.formService.updateBescheidDocumentFile(uri);
    }
  }

  public uploadFile(file: File): void {
    this.uploadDocumentFileInProgress$ = this.bescheidService.uploadBescheidDocument(
      this.bescheidDraftStateResource.resource,
      file,
    );
  }

  ngOnDestroy(): void {
    if (isNotNil(this.documentUriChangeSubscription))
      this.documentUriChangeSubscription.unsubscribe();
  }
}
