/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { faker } from '@faker-js/faker';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';
import { WiedervorlageListLinkRel } from '../src/lib/wiedervorlage.linkrel';
import {
  Wiedervorlage,
  WiedervorlageListResource,
  WiedervorlageResource,
} from '../src/lib/wiedervorlage.model';

const past = new Date();
const future = new Date();

past.setDate(past.getDate() - 2);
future.setDate(future.getDate() + 2);

export function createWiedervorlage(): Wiedervorlage {
  return {
    done: faker.datatype.boolean(),
    betreff: faker.lorem.words(3),
    beschreibung: faker.lorem.words(12),
    frist: faker.date.between({ from: past, to: future }),
    createdAt: faker.date.past(),
    attachments: faker.internet.url(),
  };
}

export function createWiedervorlageResource(linkRelations: string[] = []): WiedervorlageResource {
  return toResource(createWiedervorlage(), linkRelations);
}

export function createWiedervorlageResources(
  linkRelations: string[] = [],
): WiedervorlageResource[] {
  return times(10, () => toResource(createWiedervorlageResource(), [...linkRelations]));
}

export function createWiedervorlageListResource(
  linkRelations: string[] = [],
): WiedervorlageListResource {
  return toResource({}, [...linkRelations], {
    [WiedervorlageListLinkRel.WIEDERVORLAGE_LIST]: createWiedervorlageResources(),
  });
}
