import { EMPTY_ARRAY, getEmbeddedResources, StateResource } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { InstantSearchResult } from '@ods/system';
import { isNull } from 'lodash-es';
import { map, Observable } from 'rxjs';
import { ZustaendigeStelleService } from '../zustaendige-stelle.service';
import { ExterneFachstelleResourceSearchService } from './externe-fachstelle-resource-search.service';
import { ExterneFachstelleListLinkRel } from './externe-fachstelle.linkrel';
import { ExterneFachstelleListResource, ExterneFachstelleResource } from './externe-fachstelle.model';

@Injectable()
export class ExterneFachstelleService implements ZustaendigeStelleService<ExterneFachstelleResource> {
  constructor(private readonly searchService: ExterneFachstelleResourceSearchService) {}

  public getSearchResultList(): Observable<InstantSearchResult<ExterneFachstelleResource>[]> {
    return this.searchService
      .getResultList()
      .pipe(
        map((externeFachstelleStateListResource: StateResource<ExterneFachstelleListResource>) =>
          this.mapListResource(externeFachstelleStateListResource),
        ),
      );
  }

  mapListResource(
    externeFachstelleStateListResource: StateResource<ExterneFachstelleListResource>,
  ): InstantSearchResult<ExterneFachstelleResource>[] {
    return this.getExterneFachstellenResources(externeFachstelleStateListResource).map(
      (externeFachstelle: ExterneFachstelleResource) => this.mapToInstantSearchResult(externeFachstelle),
    );
  }

  private getExterneFachstellenResources(
    externeFachstelleStateListResource: StateResource<ExterneFachstelleListResource>,
  ): ExterneFachstelleResource[] {
    const resources: ExterneFachstelleResource[] = getEmbeddedResources(
      externeFachstelleStateListResource,
      ExterneFachstelleListLinkRel.EXTERNE_FACHSTELLE_LIST,
    );
    return isNull(resources) ? EMPTY_ARRAY : resources;
  }

  mapToInstantSearchResult(externeFachstelle: ExterneFachstelleResource): InstantSearchResult<ExterneFachstelleResource> {
    return {
      title: externeFachstelle.name,
      description: externeFachstelle.anschrift,
      data: externeFachstelle,
    };
  }

  public search(searchBy: string): void {
    this.searchService.search(searchBy);
  }

  public clearSearchResult(): void {
    this.searchService.clearResultList();
  }

  public getSelectedResult(): Observable<ExterneFachstelleResource> {
    return this.searchService.getSelectedResult();
  }

  public selectSearchResult(externeFachstelle: ExterneFachstelleResource): void {
    this.searchService.selectResult(externeFachstelle);
  }

  public clearSelectedResult(): void {
    this.searchService.clearSelectedResult();
  }
}
