import { isNotNull } from '@alfa-client/tech-shared';
import {
  ExterneFachstelleResource,
  ExterneFachstelleService,
  ZUSTAENDIGE_STELLE_SERVICE,
  ZustaendigeStelleService,
} from '@alfa-client/zustaendige-stelle-shared';
import { Component, Inject, Input, OnDestroy, OnInit } from '@angular/core';
import { AbstractControl } from '@angular/forms';
import { ResourceUri, getUrl } from '@ngxp/rest';
import { Observable, tap } from 'rxjs';

@Component({
  selector: 'alfa-search-externe-fachstelle-container',
  templateUrl: './search-externe-fachstelle-container.component.html',
  providers: [{ provide: ZUSTAENDIGE_STELLE_SERVICE, useClass: ExterneFachstelleService }],
})
export class SearchExterneFachstelleContainerComponent implements OnInit, OnDestroy {
  @Input() public fieldControl: AbstractControl<ResourceUri>;

  public externeFachstelleResource$: Observable<ExterneFachstelleResource>;

  constructor(
    @Inject(ZUSTAENDIGE_STELLE_SERVICE)
    private service: ZustaendigeStelleService<ExterneFachstelleResource>,
  ) {}

  ngOnInit(): void {
    this.externeFachstelleResource$ = this.getSelectedResult();
  }

  ngOnDestroy(): void {
    this.service.clearSelectedResult();
  }

  getSelectedResult(): Observable<ExterneFachstelleResource> {
    return this.service
      .getSelectedResult()
      .pipe(tap((externeFachstelleResource: ExterneFachstelleResource) => this.handleResult(externeFachstelleResource)));
  }

  handleResult(externeFachstelleResource: ExterneFachstelleResource): void {
    if (isNotNull(externeFachstelleResource)) {
      this.fieldControl.patchValue(getUrl(externeFachstelleResource));
    }
  }
}
