import { Mock, existsAsHtmlElement, getMockComponent, mock } from '@alfa-client/test-utils';
import {
  OrganisationsEinheitResource,
  OrganisationsEinheitService,
  ZUSTAENDIGE_STELLE_SERVICE,
} from '@alfa-client/zustaendige-stelle-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl } from '@angular/forms';
import { getUrl } from '@ngxp/rest';
import { PublicAdministrationIconComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createOrganisationsEinheitResource } from 'libs/zustaendige-stelle-shared/test/organisations-einheit';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { EditZustaendigeStelleButtonComponent } from '../edit-zustaendige-stelle-button/edit-zustaendige-stelle-button.component';
import { ZustaendigeStelleButtonComponent } from '../zustaendige-stelle-button/zustaendige-stelle-button.component';
import { ZustaendigeStelleHeaderComponent } from '../zustaendige-stelle-header/zustaendige-stelle-header.component';
import { OrganisationsEinheitComponent } from './organisations-einheit/organisations-einheit.component';
import { SearchOrganisationsEinheitContainerComponent } from './search-organisations-einheit-container.component';

describe('SearchOrganisationsEinheitContainerComponent', () => {
  let component: SearchOrganisationsEinheitContainerComponent;
  let fixture: ComponentFixture<SearchOrganisationsEinheitContainerComponent>;

  const organisationsEinheitComp: string = getDataTestIdOf('organisations-einheit-in-collaboration');

  const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

  let organisationsEinheitService: Mock<OrganisationsEinheitService>;

  beforeEach(async () => {
    organisationsEinheitService = {
      ...mock(OrganisationsEinheitService),
      getSelectedResult: jest.fn().mockReturnValue(of(createOrganisationsEinheitResource())),
    };
    TestBed.overrideComponent(SearchOrganisationsEinheitContainerComponent, {
      set: {
        providers: [
          {
            provide: ZUSTAENDIGE_STELLE_SERVICE,
            useValue: organisationsEinheitService,
          },
        ],
      },
    });
    await TestBed.configureTestingModule({
      declarations: [
        SearchOrganisationsEinheitContainerComponent,
        MockComponent(OrganisationsEinheitComponent),
        MockComponent(ZustaendigeStelleButtonComponent),
        MockComponent(ZustaendigeStelleHeaderComponent),
        MockComponent(PublicAdministrationIconComponent),
        MockComponent(EditZustaendigeStelleButtonComponent),
      ],
      providers: [
        {
          provide: OrganisationsEinheitService,
          useValue: organisationsEinheitService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(SearchOrganisationsEinheitContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInt', () => {
    beforeEach(() => {
      component.getSelectedResult = jest.fn().mockReturnValue(of(organisationsEinheitResource));
    });

    it('should call getSelectedResult', () => {
      component.ngOnInit();

      expect(component.getSelectedResult).toHaveBeenCalled();
    });

    it('should set organisationsEinheitResource', (done) => {
      component.ngOnInit();

      component.organisationsEinheitResource$.subscribe((organisationsEinheitResource: OrganisationsEinheitResource) => {
        expect(organisationsEinheitResource).toBe(organisationsEinheitResource);
        done();
      });
    });
  });

  describe('ngOnDestory', () => {
    it('should clear selected result', () => {
      component.ngOnDestroy();

      expect(organisationsEinheitService.clearSelectedResult).toHaveBeenCalled();
    });
  });

  describe('get selected result', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

    beforeEach(() => {
      organisationsEinheitService.getSelectedResult.mockReturnValue(of(organisationsEinheitResource));
    });

    it('should call service', () => {
      component.getSelectedResult().subscribe();

      expect(organisationsEinheitService.getSelectedResult).toHaveBeenCalled();
    });

    it('should call handleResult', () => {
      component.handleResult = jest.fn();

      component.getSelectedResult().subscribe();

      expect(component.handleResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should return value', (done) => {
      component.handleResult = jest.fn();

      component.getSelectedResult().subscribe((result) => {
        expect(result).toBe(organisationsEinheitResource);
        done();
      });
    });
  });

  describe('handle result', () => {
    beforeEach(() => {
      component.fieldControl = new FormControl();
    });

    it('should patch fieldControl with resource uri', () => {
      const fieldControlPatchSpy: jest.SpyInstance = (component.fieldControl.patchValue = jest.fn());

      component.handleResult(organisationsEinheitResource);

      expect(fieldControlPatchSpy).toHaveBeenCalledWith(getUrl(organisationsEinheitResource));
    });

    it('should not patch fieldControl if organisationsEinheit resource is null', () => {
      const fieldControlPatchSpy: jest.SpyInstance = (component.fieldControl.patchValue = jest.fn());

      component.handleResult(null);

      expect(fieldControlPatchSpy).not.toHaveBeenCalled();
    });
  });

  describe('organisationsEinheit component', () => {
    beforeEach(() => {
      component.organisationsEinheitResource$ = of(organisationsEinheitResource);
    });

    it('should be visible on existing organisationsEinheit', () => {
      fixture.detectChanges();

      existsAsHtmlElement(fixture, organisationsEinheitComp);
    });

    it('should be called with resource', () => {
      fixture.detectChanges();

      const comp: OrganisationsEinheitComponent = getMockComponent<OrganisationsEinheitComponent>(
        fixture,
        OrganisationsEinheitComponent,
      );
      expect(comp.organisationsEinheitResource).toBe(organisationsEinheitResource);
    });
  });
});
