/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.http.CacheControl;
import org.springframework.web.servlet.config.annotation.AsyncSupportConfigurer;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.resource.PathResourceResolver;

import lombok.NoArgsConstructor;

@Configuration
public class WebConfig implements WebMvcConfigurer {

	private static final String RESOURCE_LOCATION = "classpath:/META-INF/resources/";

	@Autowired
	private AsyncTaskExecutor threadPoolTaskExecutor;

	@Override
	public void addResourceHandlers(ResourceHandlerRegistry registry) {
		registry.addResourceHandler("/*.js", "/*.css", "/*.ttf", "/*.woff", "/*.woff2", "/*.eot",
				"/**/*.svg", "/*.svf", "/*.otf", "/*.ico", "/**/*.png")
				.addResourceLocations(RESOURCE_LOCATION)
				.setCacheControl(CacheControl.maxAge(30, TimeUnit.DAYS).cachePublic())
				.resourceChain(true)
				.addResolver(new PathResourceResolver());

		registry.addResourceHandler("/**/*")
				.addResourceLocations(RESOURCE_LOCATION)
				.setCacheControl(CacheControl.noStore())
				.setUseLastModified(false)
				.resourceChain(true)
				.addResolver(new OzgCloudPathResourceResolver());
	}

	@Override
	public void configureAsyncSupport(AsyncSupportConfigurer configurer) {
		configurer.setTaskExecutor(threadPoolTaskExecutor);
	}

	@NoArgsConstructor
	static class OzgCloudPathResourceResolver extends PathResourceResolver {

		@Override
		protected Resource getResource(String resourcePath, Resource location) throws IOException {
			var requestedResource = location.createRelative(resourcePath);

			if (requestedResource.exists() && requestedResource.isReadable()) {
				return requestedResource;
			}
			return super.getResource("index.html", location);
		}
	}
}