package de.ozgcloud.alfa.bescheid;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import org.springframework.hateoas.EntityModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(BescheidCommandController.PATH)
@RequiredArgsConstructor
class BescheidCommandController {

	static final String PATH = "/api/vorgangs/{vorgangId}/bescheids/{bescheidId}/{bescheidVersion}/commands"; // NOSONAR

	private final BescheidService bescheidService;
	private final CommandService commandService;
	private final BescheidSendenCommandValidator bescheidSendenCommandValidator;

	@PostMapping
	public ResponseEntity<EntityModel<Command>> createCommand(@PathVariable String vorgangId, @PathVariable String bescheidId,
			@PathVariable long bescheidVersion, @RequestBody CreateCommand command) {
		command = command.toBuilder().vorgangId(vorgangId).relationId(bescheidId).build();

		validate(vorgangId, command);

		var created = commandService.createCommand(command, bescheidVersion);
		return ResponseEntity.created(linkTo(CommandController.class).slash(created.getId()).toUri()).build();
	}

	void validate(String vorgangId, CreateCommand command) {
		var bescheid = bescheidService.getBescheidDraft(vorgangId)
				.orElseThrow(() -> new TechnicalException("BescheidDraft not found"));
		bescheidSendenCommandValidator.validate(command.toBuilder().body(bescheid).build());
	}
}
