package de.ozgcloud.alfa.bescheid;

import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;

import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class BescheidService {

	private final BescheidRemoteService remoteService;
	private final BescheidByBeschiedenAmComparator comparator;

	public Optional<Bescheid> getBescheidDraft(String vorgangId) {
		return remoteService.getBescheidDraft(vorgangId);
	}

	public Stream<Bescheid> findByVorgangId(String vorgangId) {
		return remoteService.findByVorgangId(vorgangId);
	}

	public Stream<Bescheid> findByVorgangIdSorted(String vorgangId) {
		return sortBescheids(remoteService.findByVorgangId(vorgangId));
	}

	Stream<Bescheid> sortBescheids(Stream<Bescheid> bescheids) {
		return bescheids.sorted(comparator.reversed());
	}

	public boolean canCreateBescheidDocumentAutomatically() {
		return remoteService.canCreateBescheidDocument();
	}

	public boolean existsBescheid(String vorgangId) {
		var bescheide = remoteService.findByVorgangId(vorgangId).toList();

		return CollectionUtils.isNotEmpty(bescheide);
	}
}
