/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.file;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.vorgang.grpc.file.FileServiceGrpc.FileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsRequest;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class OzgFileRemoteService {

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private FileServiceBlockingStub fileServiceStub;
	@Autowired
	private OzgFileMapper fileMapper;

	public Stream<OzgFile> getAttachments(String vorgangId) {
		var response = fileServiceStub.getAttachments(buildGrpcGetAttachmentsRequest(vorgangId));

		return response.getFileList().stream().map(fileMapper::toFile);
	}

	private GrpcGetAttachmentsRequest buildGrpcGetAttachmentsRequest(String vorgangId) {
		return GrpcGetAttachmentsRequest.newBuilder()
				.setVorgangId(vorgangId)
				.build();
	}

	public Stream<OzgFile> getRepresentations(String vorgangId) {
		var response = fileServiceStub.getRepresentations(buildGrpcGetRepresentationsRequest(vorgangId));

		return response.getFileList().stream().map(fileMapper::toFile);
	}

	private GrpcGetRepresentationsRequest buildGrpcGetRepresentationsRequest(String vorgangId) {
		return GrpcGetRepresentationsRequest.newBuilder()
				.setVorgangId(vorgangId)
				.build();
	}
}