package de.ozgcloud.alfa.common.user;

import static de.ozgcloud.alfa.common.GrpcUtil.*;
import static de.ozgcloud.alfa.common.callcontext.ContextService.*;

import java.util.Objects;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;

import de.ozgcloud.alfa.RequestAttributes;
import io.grpc.CallOptions;
import io.grpc.Channel;
import io.grpc.ClientCall;
import io.grpc.ClientInterceptor;
import io.grpc.ForwardingClientCall.SimpleForwardingClientCall;
import io.grpc.Metadata;
import io.grpc.MethodDescriptor;
import lombok.extern.log4j.Log4j2;

@Log4j2
class UserCallContextAttachingInterceptor implements ClientInterceptor {

	@Autowired
	private ApplicationContext context;
	@Autowired(required = false)
	private RequestAttributes reqAttributes;

	@Override
	public <A, B> ClientCall<A, B> interceptCall(MethodDescriptor<A, B> method, CallOptions callOptions, Channel next) {
		return new CallContextAttachingClientCall<>(next.newCall(method, callOptions));
	}

	final class CallContextAttachingClientCall<A, B> extends SimpleForwardingClientCall<A, B> {

		protected CallContextAttachingClientCall(ClientCall<A, B> delegate) {
			super(delegate);
		}

		@Override
		public void start(Listener<B> responseListener, Metadata headers) {
			headers.merge(buildCallContextWithoutUser());
			super.start(responseListener, headers);
		}

		private Metadata buildCallContextWithoutUser() {
			var metadata = new Metadata();

			Optional.ofNullable(context.getId()).map(String::getBytes)
					.ifPresentOrElse(bytes -> metadata.put(createKeyOf(KEY_CLIENT_NAME), bytes), () -> LOG.warn("Missing value 'client name'"));

			metadata.put(createKeyOf(KEY_REQUEST_ID), getRequestId().getBytes());

			return metadata;
		}

		private String getRequestId() {
			return Objects.isNull(reqAttributes) ? "- no request scope -" : reqAttributes.getRequestId();
		}
	}
}
