package de.ozgcloud.alfa.common.user;

import static de.ozgcloud.alfa.common.user.UserRole.*;

import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.expression.method.DefaultMethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionHandler;
import org.springframework.security.access.hierarchicalroles.RoleHierarchy;
import org.springframework.security.access.hierarchicalroles.RoleHierarchyImpl;

import io.grpc.Channel;
import io.grpc.ManagedChannelBuilder;

@Configuration
class UserConfiguration {

	@ConditionalOnProperty(name = "grpc.client.user-manager.address")
	@Bean
	Channel userManagerChannel(UserManagerClientProperties properties) {
		var builder = ManagedChannelBuilder.forTarget(properties.getAddress())
				.intercept(userCallContextAttachingInterceptor());
		if (StringUtils.equalsIgnoreCase(properties.getNegotiationType(), "PLAINTEXT")) {
			builder.usePlaintext();
		}
		return builder.build();
	}

	@Bean
	UserCallContextAttachingInterceptor userCallContextAttachingInterceptor() {
		return new UserCallContextAttachingInterceptor();
	}

	@Bean
	static RoleHierarchy roleHierarchy() {
		var hierarchy = new RoleHierarchyImpl();
		hierarchy.setHierarchy(addRolePrefix(VERWALTUNG_LOESCHEN) + " > " + addRolePrefix(VERWALTUNG_USER));
		return hierarchy;
	}

	private static String addRolePrefix(String roleName) {
		return CurrentUserHelper.ROLE_PREFIX + roleName;
	}

	@Bean
	static MethodSecurityExpressionHandler methodSecurityExpressionHandler(RoleHierarchy roleHierarchy) {
		DefaultMethodSecurityExpressionHandler expressionHandler = new DefaultMethodSecurityExpressionHandler();
		expressionHandler.setRoleHierarchy(roleHierarchy);
		return expressionHandler;
	}
}
