package de.ozgcloud.alfa.common.zipdownload;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class ZipDownloadService {

	private final BinaryFileService binaryFileService;

	public void write(List<OzgFile> ozgFiles, OutputStream out) {
		writeZipFileContent(createZipFile(ozgFiles), out);
	}

	File createZipFile(List<OzgFile> ozgFiles) {
		var file = TempFileUtils.createTmpFile().toFile();
		try (var zipOutputStream = new ZipOutputStream(new FileOutputStream(file))) {
			ozgFiles.forEach(ozgFile -> addOzgFileToZip(ozgFile, zipOutputStream));
			return file;
		} catch (Exception e) {
			throw new TechnicalException("Error creating zip file", e);
		}
	}

	void addOzgFileToZip(OzgFile ozgFile, ZipOutputStream zipOutputStream) {
		try {
			zipOutputStream.putNextEntry(new ZipEntry(ozgFile.getName()));
			binaryFileService.writeFileContent(ozgFile.getId(), zipOutputStream);
			zipOutputStream.closeEntry();
		} catch (IOException e) {
			throw new TechnicalException("Cannot add file to ZIP.", e);
		}
	}

	void writeZipFileContent(File zipFile, OutputStream outputStream) {
		try (var fileInputStream = new FileInputStream(zipFile)) {
			fileInputStream.transferTo(outputStream);
		} catch (Exception e) {
			throw new TechnicalException("Error writing zip file to output stream", e);
		}
	}

}
