package de.ozgcloud.alfa.historie;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
class UserProfileCache {

	private final Function<UserId, UserProfile> findUserProfileForUserId;

	private final Map<UserId, UserProfile> cache = new HashMap<>();

	public static UserProfileCache create(Function<UserId, UserProfile> findUserProfileForUserId) {
		return new UserProfileCache(findUserProfileForUserId);
	}

	public UserProfile getUserProfile(UserId userId) {
		if (!cache.containsKey(userId)) {
			findAndAddToCache(userId);
		}
		return cache.get(userId);
	}

	private void findAndAddToCache(UserId userId) {
		var userProfile = findUserProfileForUserId.apply(userId);
		if (userProfile == null) {
			throw new TechnicalException("UserProfile not found for user with ID " + userId);
		}
		cache.put(userId, userProfile);
	}
}
