package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import com.thedeanda.lorem.LoremIpsum;

class FachstelleServiceTest {

	@InjectMocks
	private FachstelleService service;

	@Mock
	private FachstelleRemoteService remoteService;

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(FachstelleTestFactory.ID);

			verify(remoteService).getById(FachstelleTestFactory.ID);
		}

		@Test
		void shouldReturnFachstelle() {
			var fachstelle = FachstelleTestFactory.create();
			when(remoteService.getById(FachstelleTestFactory.ID)).thenReturn(fachstelle);

			var gotFachstelle = service.getById(FachstelleTestFactory.ID);

			assertThat(gotFachstelle).isEqualTo(fachstelle);
		}
	}

	@Nested
	class TestSearchFachstellen {

		private final String searchBy = LoremIpsum.getInstance().getWords(5);

		@Test
		void shouldCallRemoteService() {
			callService();

			verify(remoteService).search(searchBy);
		}

		@Test
		void shouldReturnFachstellen() {
			var fachstelle = FachstelleTestFactory.create();
			when(remoteService.search(searchBy)).thenReturn(Stream.of(fachstelle));

			var fachstellen = callService();

			assertThat(fachstellen).containsExactly(fachstelle);
		}

		private Stream<Fachstelle> callService() {
			return service.searchFachstellen(searchBy);
		}
	}

}