package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;

public class PostfachConfigMapperTest {

	@Spy
	private final PostfachConfigMapper mapper = Mappers.getMapper(PostfachConfigMapper.class);

	@Nested
	class TestToPostfachConfig {

		@Test
		void shouldMapReplyAllowed() {
			var postfachConfig = mapper.toPostfachConfig(GrpcPostfachTestFactory.create());

			assertThat(postfachConfig.isReplyAllowed()).isEqualTo(GrpcPostfachTestFactory.REPLY_ALLOWED);
		}

		@Test
		void shouldMapType() {
			var postfachConfig = mapper.toPostfachConfig(GrpcPostfachTestFactory.create());

			assertThat(postfachConfig.getType()).isEqualTo(GrpcPostfachTestFactory.TYPE);
		}
	}

	@Nested
	class TestToPostfachConfigGroup {

		@Test
		void shouldMapConfigured() {
			var postfachConfigGroup = mapper.toPostfachConfigGroup(GrpcGetPostfachConfigResponseTestFactory.create());

			assertThat(postfachConfigGroup.isConfigured()).isEqualTo(GrpcGetPostfachConfigResponseTestFactory.CONFIGURED);
		}

		@Test
		void shouldMapPostfachConfigs() {
			var expectedPostfachConfig = PostfachConfigTestFactory.create();
			doReturn(expectedPostfachConfig).when(mapper).toPostfachConfig(GrpcPostfachTestFactory.create());

			var postfachConfigGroup = mapper.toPostfachConfigGroup(GrpcGetPostfachConfigResponseTestFactory.create());

			assertThat(postfachConfigGroup.getPostfachConfigs()).containsOnly(expectedPostfachConfig);
		}
	}
}
