/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.alfa.CallScope;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.EingangTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.nachrichten.postfach.PostfachServiceGrpc.PostfachServiceBlockingStub;

@SpringBootTest
public class PostfachMailITCase {

	@Autowired
	private PostfachMailController controller;

	@SpyBean
	private PostfachMailRemoteService postfachRemoteService;
	@Mock
	private PostfachServiceBlockingStub serviceStub;
	@MockBean
	private VorgangController vorgangController;

	@SpyBean
	private CurrentUserService userService;

	@Autowired
	private CallScope callScope;

	@BeforeEach
	void initTest() {
		ReflectionTestUtils.setField(postfachRemoteService, "serviceStub", serviceStub);

		callScope.startScope();
	}

	@Nested
	@WithMockUser
	class TestGetAll {

		@BeforeEach
		void mock() {
			when(serviceStub.findPostfachMails(any())).thenReturn(GrpcPostfachMailTestFactory.createFindPostfachMailsResponse());
			when(serviceStub.getPostfachConfig(any())).thenReturn(GrpcGetPostfachConfigResponseTestFactory.create());
			when(vorgangController.getVorgang(anyString())).thenReturn(
					VorgangWithEingangTestFactory.createBuilder().eingang(EingangTestFactory.createBuilder().antragsteller(null).build()).build());

			doReturn(UserProfileTestFactory.ID).when(userService).getUserId();
		}

		@Test
		void shouldProceedWithoutAntragsteller() {
			controller.getAll(VorgangHeaderTestFactory.ID);

			verify(vorgangController).getVorgang(VorgangHeaderTestFactory.ID);
			verify(postfachRemoteService).findPostfachMails(VorgangHeaderTestFactory.ID);
		}
	}
}