package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;

class VorgangURIResolverTest {

	private VorgangURIResolver resolver = new VorgangURIResolver();

	@Nested
	class TestResolve {

		private static final String VORGANG_ID = VorgangHeaderTestFactory.ID;

		@ParameterizedTest
		@MethodSource("invalidVorgangUris")
		void shouldReturnEmptyIfUriIsNotRecognized(String uri) {
			var result = resolver.resolve(uri);

			assertThat(result).isEmpty();
		}

		@ParameterizedTest
		@MethodSource("validVorgangUris")
		void shouldReturnLink(String uri) {
			var result = resolver.resolve(uri);

			assertThat(result).isPresent();
		}

		@Nested
		class TestVorgangLink {

			@ParameterizedTest
			@MethodSource("de.ozgcloud.alfa.vorgang.VorgangURIResolverTest$TestResolve#validVorgangUris")
			void shouldHaveCorrectRelValue(String uri) {
				var result = resolver.resolve(uri);

				assertThat(result).get().extracting(Link::getRel).extracting(LinkRelation::value).isEqualTo(VorgangURIResolver.REL_NAME);
			}

			@ParameterizedTest
			@MethodSource("de.ozgcloud.alfa.vorgang.VorgangURIResolverTest$TestResolve#validVorgangUris")
			void shouldHaveHrefOfVorgang(String uri) {
				var result = resolver.resolve(uri);

				assertThat(result).get().extracting(Link::getHref).isEqualTo("/api/vorgangs/" + VORGANG_ID);
			}
		}

		private static Stream<String> validVorgangUris() {
			return Stream.of(
					"dns:///dummy.service/vorgangs/" + VORGANG_ID,
					"dns:///dummy.service:123/vorgangs/" + VORGANG_ID,
					"http://dummy.web.url/vorgangs/" + VORGANG_ID);
		}

		private static Stream<String> invalidVorgangUris() {
			return Stream.of("dns:///dummy.service/res-a/" + VORGANG_ID,
					"dns:///dummy.service/" + VORGANG_ID,
					"dns:///dummy.service/vorgangs/xyz/" + VORGANG_ID,
					"dns:///dummy.service/vorgangs/" + VORGANG_ID + "/" + VORGANG_ID);
		}
	}
}
