/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.wiedervorlage.WiedervorlageCommandController.WiedervorlageCommandByVorgangController;
import lombok.SneakyThrows;

class WiedervorlageCommandByVorgangControllerTest {

	@Spy
	@InjectMocks
	private WiedervorlageCommandByVorgangController controller;

	@Mock
	private WiedervorlageService service;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@DisplayName("Create command")
	@Nested
	class TestCreateCommand {

		@Captor
		private ArgumentCaptor<Wiedervorlage> wiedervorlageCaptor;

		@BeforeEach
		void mockUserService() {
			when(service.createWiedervorlage(any(), any())).thenReturn(CommandTestFactory.create());
		}

		@Nested
		class ServiceMethod {

			@Test
			void shouldCallService() {
				doRequest();

				verify(service).createWiedervorlage(wiedervorlageCaptor.capture(), eq(VorgangHeaderTestFactory.ID));
				assertThat(wiedervorlageCaptor.getValue().getAttachments().getFirst()).isEqualTo(BinaryFileTestFactory.FILE_ID);
			}

			@Test
			void shouldCallServiceToUpdateNextFrist() {
				doRequest();

				verify(service).updateNextFrist(eq(VorgangHeaderTestFactory.ID), wiedervorlageCaptor.capture());
				assertThat(wiedervorlageCaptor.getValue())
						.usingRecursiveComparison()
						.comparingOnlyFields("betreff", "beschreibung", "frist", "attachments")
						.isEqualTo(WiedervorlageTestFactory.create());
			}

			@SneakyThrows
			@Test
			void returnReturnCreated() {
				doRequest().andExpect(status().isCreated());
			}
		}
	}

	@SneakyThrows
	private ResultActions doRequest() {
		return mockMvc.perform(
						post(WiedervorlageCommandByVorgangController.WIEDERVORLAGE_COMMANDS_BY_VORGANG,
								VorgangHeaderTestFactory.ID)
								.content(createRequestContent())
								.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().is2xxSuccessful());
	}

	private String createRequestContent() {
		var createCommand = CommandTestFactory.createCreateCommandBuilder().body(WiedervorlageTestFactory.create())
				.order(LegacyOrder.EDIT_WIEDERVORLAGE)
				.build();
		return WiedervorlageTestFactory.createRequestContent(createCommand);
	}
}