package de.ozgcloud.alfa.file;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.alfa.common.VersionTypeBuilder;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.MediumCodeType;
import de.xoev.xdomea.VersionType;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class DokumentTypeBuilder {

	static final String VERSION_NUMMER = "1";
	static final String ALLGEMEINE_METADATEN_MEDIUM_CODE = "001";

	private Long ordinalNumber;
	private String formEngineName;
	private OzgFile ozgFile;

	public static DokumentTypeBuilder builder() {
		return new DokumentTypeBuilder();
	}

	public DokumentTypeBuilder withOrdinalNumber(Long ordinalNumber) {
		this.ordinalNumber = ordinalNumber;
		return this;
	}

	public DokumentTypeBuilder withFormEngineName(String formEngineName) {
		this.formEngineName = formEngineName;
		return this;
	}

	public DokumentTypeBuilder withOzgFile(OzgFile ozgFile) {
		this.ozgFile = ozgFile;
		return this;
	}

	public DokumentType build() {
		var dokumentType = new DokumentType();
		dokumentType.setIdentifikation(createIdentifikation());
		dokumentType.setAllgemeineMetadaten(createAllgemeineMetadaten());
		dokumentType.getVersion().add(createVersionType());
		return dokumentType;
	}

	VersionType createVersionType() {
		return VersionTypeBuilder.builder()
				.withOzgFile(ozgFile)
				.withErsteller(formEngineName)
				.withSonstigerName(StringUtils.EMPTY)
				.build();
	}

	IdentifikationObjektType createIdentifikation() {
		return IdentifikationObjektTypeBuilder.builder()
				.withObjectID(ozgFile.getId().toString())
				.withOrdinalNumber(ordinalNumber)
				.build();
	}

	AllgemeineMetadatenType createAllgemeineMetadaten() {
		var allgemeineMetadaten = new AllgemeineMetadatenType();
		allgemeineMetadaten.setBetreff(ozgFile.getName());
		allgemeineMetadaten.setKennzeichen(StringUtils.EMPTY);
		allgemeineMetadaten.setBemerkung(StringUtils.EMPTY);
		allgemeineMetadaten.setMedium(createMediumCode());
		return allgemeineMetadaten;
	}

	private MediumCodeType createMediumCode() {
		var mediumCode = new MediumCodeType();
		mediumCode.setCode(ALLGEMEINE_METADATEN_MEDIUM_CODE);
		return mediumCode;
	}
}
