package de.ozgcloud.alfa.vorgang;

import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.MediumCodeType;
import de.xoev.xdomea.VorgangType;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class VorgangTypeCreator {

	static final Long NUMMER_IM_UEBERGEORDNETEN_CONTAINER = 1L;
	static final String ALLGEMEINE_METADATEN_MEDIUM_CODE = "001";

	private final AnwendungsspezifischeErweiterungTypeCreator anwendungsspezifischeErweiterungTypeCreator;
	private final AnwendungsspezifischeErweiterungXMLTypeCreator anwendungsspezifischeErweiterungXMLTypeCreator;
	private final KontaktTypeCreator kontaktTypeCreator;

	public VorgangType create(VorgangWithEingang vorgangWithEingang) {
		var vorgang = new VorgangType();
		vorgang.setAnwendungsspezifischeErweiterung(anwendungsspezifischeErweiterungTypeCreator.create(vorgangWithEingang));
		vorgang.setAnwendungsspezifischeErweiterungXML(anwendungsspezifischeErweiterungXMLTypeCreator.create(vorgangWithEingang));
		vorgang.setIdentifikation(createIdentifikation());
		vorgang.setAllgemeineMetadaten(createAllgemeineMetadaten(vorgangWithEingang));
		kontaktTypeCreator.create(vorgangWithEingang).ifPresent(vorgang.getKontakt()::add);
		return vorgang;
	}

	IdentifikationObjektType createIdentifikation() {
		var identifikation = new IdentifikationObjektType();
		identifikation.setID(UUID.randomUUID().toString());
		identifikation.setNummerImUebergeordnetenContainer(NUMMER_IM_UEBERGEORDNETEN_CONTAINER);
		return identifikation;
	}

	AllgemeineMetadatenType createAllgemeineMetadaten(VorgangWithEingang vorgang) {
		var allgemeineMetadaten = new AllgemeineMetadatenType();
		allgemeineMetadaten.setBetreff(vorgang.getName());
		allgemeineMetadaten.setKennzeichen(vorgang.getNummer());
		allgemeineMetadaten.setBemerkung(StringUtils.EMPTY);
		allgemeineMetadaten.setMedium(createMediumCode());
		return allgemeineMetadaten;
	}

	private MediumCodeType createMediumCode() {
		var mediumCode = new MediumCodeType();
		mediumCode.setCode(ALLGEMEINE_METADATEN_MEDIUM_CODE);
		return mediumCode;
	}

}
