package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullSource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;

import de.xoev.xdomea.DatatypeType;

class DatatypeMapperTest {

	@InjectMocks
	private DatatypeMapper datatypeMapper;

	@DisplayName("should return DatatypeType STRING when")
	@ParameterizedTest(name = "\"{0}\"")
	@NullSource
	@ValueSource(strings = { StringUtils.EMPTY, "a" })
	void shouldReturnString(String arg) {
		var datatype = datatypeMapper.from(arg);

		assertThat(datatype).isEqualTo(DatatypeType.STRING);
	}

	@Test
	void shouldReturnStringAsDefault() {
		var datatype = datatypeMapper.from(new Object());

		assertThat(datatype).isEqualTo(DatatypeType.STRING);
	}

	@Test
	void shouldReturnDate() {
		var datatype = datatypeMapper.from(LocalDate.now());

		assertThat(datatype).isEqualTo(DatatypeType.DATE);
	}

	@Test
	void shouldReturnDatetime() {
		var datatype = datatypeMapper.from(ZonedDateTime.now());

		assertThat(datatype).isEqualTo(DatatypeType.DATETIME);
	}

	@Test
	void shouldReturnInteger() {
		var datatype = datatypeMapper.from(1);

		assertThat(datatype).isEqualTo(DatatypeType.INTEGER);
	}

	@Test
	void shouldReturnFloat() {
		var datatype = datatypeMapper.from(1.0f);

		assertThat(datatype).isEqualTo(DatatypeType.FLOAT);
	}

	@Test
	void shouldReturnBoolean() {
		var datatype = datatypeMapper.from(true);

		assertThat(datatype).isEqualTo(DatatypeType.BOOLEAN);
	}

	@Test
	void shouldReturnTypeOfElementsInCollection() {
		var datatype = datatypeMapper.from(List.of(1, 2));

		assertThat(datatype).isEqualTo(DatatypeType.INTEGER);
	}

	@Test
	void shouldReturnDefaultIfCollectionIsEmpty() {
		var datatype = datatypeMapper.from(new Object[] {});

		assertThat(datatype).isEqualTo(DatatypeType.STRING);
	}
}